// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/PartitionManager.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:38
//
// Module  :  PartitionManager.java
//
// Source File Description:
//
//    see javadoc for this interface.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 271911       WASX      May 16, 2005 sutter    New code drop
// XD601_M2_B.2 WASX.XD   Sep 27, 2005 sutter    Decided on partitionLogSequence() method
// XD61         XD61      11/10/2006   haowang   create new partition manager
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.util.List;

import com.ibm.websphere.objectgrid.plugins.LogSequence;

/**
 * This interface will be used for calculating the proper partition for a
 * given input key.  The set of partitions is determined by the BackingMap configuration.
 *
 * @see BackingMap#getPartitionManager()
 * @since WAS XD 6.0.1, XC10
 */
public interface PartitionManager {


    /**
     * Obtains a 0-based partition number determined by the input parameter's
     * hashCode() method.
     *
     * @param key Individual key used to determine partition (can not be null)
     * @return int 0-based partition number
     */
    public abstract int getPartition(Object key);

    /**
     * Obtains the 0-based partition numbers for each of the keys in the input
     * <code>List</code> of keys.  Each object in the returned list of
     * partition identifiers is an instance of <code>java.lang.Integer</code>.
     *
     * @param keyList Ordered list of keys
     * @return List of partition identifiers that corresponds to the input list of keys
     */
    public abstract List /*<Integer>*/ getPartitions(List keyList);

    /**
     * This method is very similar to <code>getPartitions(List)</code>, except
     * it returns the keys organized by the partition identifiers.  The return
     * value is a <code>List</code> of <code>List</code>s.  The outer
     * <code>List</code> is an ordered <code>List</code> of the partition
     * numbers, with the first entry in the <code>List</code> corresponding to
     * partition 0. The inner <code>List</code>s contain the keys from the input
     * parameter that correspond to that partition identifier.
     * <p>
     * The return value will always contain a <code>List</code> object.  Either
     * the outer or the inner <code>List</code>s may contain zero elements, but
     * the <code>List</code> objects themselves will not be <code>null</code>.
     *
     * @param keyList Ordered list of keys
     * @return List of Lists that correspond to the 0-based partition numbers, with
     * each inner List containing the set of keys that parse to that partition
     * number.
     */

    public abstract List /*<List>*/ getPartitionLists(List keyList);

    /**
     * Partitions a <code>LogSequence</code> based on the partitioning
     * algorithm for the Map.
     *
     * @param ls LogSequence that needs to be partitioned
     * @return List of partitioned LogSequences.  The first LogSequence in the List
     * corresponds to the first partition, etc.
     */
    public abstract List /*<LogSequence>*/ partitionLogSequence(LogSequence ls);

    /**
     * Returns the number of configured partitions for this PartitionManager.
     *
     * @return the number of configured partitions
     */
    public abstract int getNumOfPartitions();
}