// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.5 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/TxID.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:51
//
// Module  :  TxID.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// XD60_M3_B.8  WASX.XD   04/14/2005   sutter    M3: client/server updates
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.io.Serializable;

import com.ibm.websphere.objectgrid.plugins.Loader;
import com.ibm.websphere.objectgrid.plugins.TransactionCallback;

/**
 * This interface is an opaque identifier for a transaction. Context information
 * can be stored and retrieved in multiple slots on this object. This mechanism
 * allows a <code>TransactionCallback</code> and <code>Loader</code>, for
 * example, to share state information with each other in the context of a
 * specific session transaction. <p>
 * 
 * The TxID.toString() output can be used to determine whether the originating Session
 * transaction is a single partition transaction or a multi-partition transaction. If the
 * String output begins with the keyword <code>Local</code> then this indicates a single
 * partition transaction, for example: <code>Local-40000139-72B2-C037-E000-1C271366B073</code> <p>
 * If the String output begins with the keyword <code>WXS</code> then this indicates a 
 * multi-partition transaction, for example: <code>WXS-40000139-72B2-BD3A-E000-1C271366B073</code>
 *  
 *
 * @since WAS XD 6.0, XC10
 *
 * @see Loader
 * @see ObjectGrid#reserveSlot(String)
 * @see Session
 * @see TransactionCallback
 */
public interface TxID extends Serializable
{
    /**
     * All slots should be reserved using this name.
     *
     * @see ObjectGrid#reserveSlot(String)
     */
    String SLOT_NAME = "IBM_TXID";

    /**
     * Checks for equality between two TxID objects.
     *
     * @param o Input TxID to check for equality against
     *
     * @return true, if they are equal; false, if they not equal
     */
    boolean equals(TxID o);

    /**
     * Returns the hashcode of the Tx identifier.
     *
     * @return hashcode
     */
    int hashCode();

    /**
     * Gets the context information currently associated with this transaction.
     *
     * @param slotNumber the slot number for the context information being requested
     *
     * @return Object the current context information for the slot number
     *
     * @see #putSlot(int, Object)
     * @see ObjectGrid#reserveSlot(String)
     */
    Object getSlot(int slotNumber);

    /**
     * Sets some context information to be associated with this transaction.
     *
     * @param slotNumber the slot number
     * @param o Object to be put into the TxID slot
     *
     * @see #getSlot(int)
     * @see ObjectGrid#reserveSlot(String)
     */
    void putSlot(int slotNumber, Object o);

    /**
     * Returns the Session that owns this TxID.
     *
     * @return a Session object to use.
     *
     * @see Session
     */
    Session getSession();
}
