// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005, 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  BackingMapConfiguration.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// b896                05/01/2006   vandersn  open config APIs
// --------- --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.config;

import java.util.List;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.TTLType;

/**
 * A BackingMapConfiguration object can be used to override BackingMap settings on the
 * client side.  The <code>com.ibm.websphere.objectgrid.plugins.Evictor</code>
 * and the <code>com.ibm.websphere.objectgrid.plugins.MapEventListener</code> Plugins can be overridden.
 * Other Evictor related settings can be adjusted as well as client, near-cache specific options.
 * <p>
 * Use the <code>com.ibm.websphere.objectgrid.config.ObjectGridConfigFactory.createBackingMapConfiguration(String)</code>
 * method to create a BackingMapConfiguration
 * @see com.ibm.websphere.objectgrid.plugins.Evictor
 * @see com.ibm.websphere.objectgrid.plugins.MapEventListener
 * @see com.ibm.websphere.objectgrid.config.Plugin
 * @see com.ibm.websphere.objectgrid.config.ObjectGridConfigFactory
 * @since WAS XD 6.0.1.2, XC10
 */

public interface BackingMapConfiguration {


    /**
     * Get the name of this BackingMapConfiguration
     * @return The name of this BackingMapConfiguration
     */
    public String getName();

    /**
     * Add a <code>Plugin</code> to this BackingMapConfiguration.  The Plugins that can be overridden on a
     * client-side BackingMap are <code>com.ibm.websphere.objectgrid.plugins.Evictor</code>
     * and <code>com.ibm.websphere.objectgrid.plugins.MapEventListener</code>.
     * @param plugin
     * @see #setPlugins(List)
     */
	public void addPlugin(Plugin plugin);

    /**
     * Set the Plugins for this BackingMapConfiguration.  Any Plugins that were previously attached to this
     * BackingMapConfiguration object will be overridden.
     * @param pluginList - a List of Plugins
     * @see #addPlugin(Plugin)
     */
	public void setPlugins(List pluginList);

    /**
     * Get the Plugins that have been attached to this BackingMapConfiguration.
     * @return a List of <code>Plugin</code> objects
     */
	public List getPlugins();



    /**
     * Gets the number of buckets defined for this BackingMapConfiguration.
     * @return the number of buckets defined
     * @deprecated Deprecated in 8.6.  Buckets are no longer required.  Use the {@link #isNearCacheEnabled()} flag to disable
     * the near cache in the ObjectGrid configuration XML file.
     */
	public int getNumberOfBuckets();

    /**
     * Sets the number of buckets for this BackingMapConfiguration.  This will be used by the
     * <code>BackingMap</code>.
     * <p>
     * The <code>BackingMap</code> implementation uses a hash map for its
     * implementation.  If there are a lot of entries in the <code>BackingMap</code>
     * then more buckets means better performance because the risk of collisions
     * is lower as the number of buckets grows. More buckets also means more
     * concurrency.
     * @param numBuckets
     * @see BackingMap#setNumberOfBuckets(int)
     * @deprecated Deprecated in 8.6.  Buckets are no longer required.  Use the {@link #isNearCacheEnabled()} flag to disable
     * the near cache in the ObjectGrid configuration XML file.
     */
	public void setNumberOfBuckets(int numBuckets);

    /**
     * Gets the "time to live" for each map entry.  The value is in seconds.
     * @return the "time to live" in seconds
     */
	public int getTimeToLive();
	/**
	 * Sets "time to live" of each BackingMap entry in seconds.
     * <p>
     * If this method is not called, the lifetime of an entry is forever (or until
     * the application explicitly removes or invalidates the entry, or a user
     * defined <code>Evictor</code> evicts the entry).
     *
     * @param seconds
	 */
	public void setTimeToLive(int seconds);

    /**
     * Gets the "time to live" Evictor type for this BackingMapConfiguration.  If setTtlEvictorType was
     * not called, this method will return null and the BackingMap based off this BackingMapConfiguration
     * will use <code>TTLType.NONE</code>
     * @return the "time to live" Evictor type or null if <code>setTtlEvictorType(TTLType)<code> was not called
     * @see #setTimeToLive(int)
     */
	public TTLType getTtlEvictorType();
	/**
     * Set the "time to live" Evictor type for this BackingMapConfiguration.
     * This is used to determine how expiration time of a <code>BackingMap</code> entry is computed.
     * <p>
     * If this method is not called, <code>TTLType.NONE</code> is used
     * to indicate the map entry has no expiration time (e.g. is allowed to live
     * until explicitly removed or invalidated by the application, or evicted
     * by a user defined <code>Evictor</code>).

     * @param ttlEvictorType
     * @see BackingMap#setTtlEvictorType(TTLType)
	 */
	public void setTtlEvictorType(TTLType ttlEvictorType);

    /**
     * Gets the list of eviction triggers for this BackingMapConfiguration.<p>
     *
     * See {@link BackingMap} for a list of valid eviction triggers.
     * @return a semicolon separated list of eviction triggers or null if <code>setEvictionTriggers(String)<code> was not called
     * @since WAS XD 6.1.0.3
     */
    public String getEvictionTriggers();

    /**
     * Sets the eviction triggers for this BackingMapConfiguration. All evictors will use the eviction
     * supplied triggers.<p>
     *
     * See {@link BackingMap} for a list of valid eviction triggers.
     * @param evictionTriggers a semicolon separated list of eviction triggers
     * @since WAS XD 6.1.0.3
     */

    public void setEvictionTriggers(String evictionTriggers);

    /**
     * If true, clients with local caches are automatically invalidated when the data grid map
     * is updated.
     * @return true if client near cache invalidation is enabled, false if client near cache invalidation
     * is disabled, or null if the override is not specified.
     * @since 8.6
     */
    public Boolean isNearCacheInvalidationEnabled();

    /**
     * Set to true to enable client near cache invalidation.  When enabled, the client
     * will receive events from the remote data grid to invalidate data from the local cache.
     * @param nearCacheInvalidationEnabled If true, the client near cache invalidation is enabled. If false, invalidation
     * is disabled.  If null, the override is not specified and the client will use the setting from the remote data grid.
     * @since 8.6
     */
    public void setNearCacheInvalidationEnabled(Boolean nearCacheInvalidationEnabled);

    /**
     * If true, clients automatically send time-to-live access information
     * to the remote data grid when accessed and the {@link TTLType#LAST_ACCESS_TIME}
     * TTL evictor type is configured.
     * @return True if last-access time-to-live information is sent to the remote data grid, false if last-access
     * time information is not sent, or null if the override is not specified.
     * @since 8.6
     */
    public Boolean isNearCacheLastAccessTTLSyncEnabled();

    /**
     * Enables or disables time-to-live access information synchronization
     * to the remote data grid when accessed and the {@link TTLType#LAST_ACCESS_TIME}
     * TTL evictor type is configured.
     * @param nearCacheLastAccessTTLSyncEnabled If true, the last-access time-to-live information is sent to the remote data grid.
     * If false, the last-access information is not sent.  If null, the override is not specified and the client will use the
     * setting from the remote data grid.
     * @since 8.6
     */
    public void setNearCacheLastAccessTTLSyncEnabled(Boolean nearCacheLastAccessTTLSyncEnabled);

    /**
     * If true, the client local cache is enabled for supported configurations.
     * The client near cache can only be enabled when using optimistic locking or
     * when locking is disabled.
     * @return True if the client near cache is enabled, false if the near cache is disabled, or null of the
     * override is not specified.
     * @since 8.6
     */
    public Boolean isNearCacheEnabled();

    /**
     * Enables or disables the client local cache for supported configurations.
     * The client near cache can only be enabled when using optimistic locking or
     * when locking is disabled.
     * @param nearCacheEnabled If true, the client local cache is enabled for supported configurations. If false, the
     * client local cache is disabled.  If null, the override is not specified and the client will use the
     * setting from the remote data grid.
     */
    public void setNearCacheEnabled(Boolean nearCacheEnabled);
    /**
     * Retrieves the data format for all data access APIs that return cache keys.
     * @return the data format or null if the default should be used.
     * @since 8.6
     */
    public OutputFormat getKeyOutputFormat();
    
    /**
     * Sets the data format for all data access APIs that return cache keys.
     * @param dataFormat the data format to use or null to use the default.
     * @since 8.6
     */
    public void setKeyOutputFormat(OutputFormat dataFormat);

    /**
     * Retrieves the data format for all data access APIs that return cache values.
     * @return the data format.
     * @since 8.6
     */
    public OutputFormat getValueOutputFormat();

    /**
     * Sets the data format for all data access APIs that return cache values.
     * @param dataFormat the data format to use or null to use the default.
     * @since 8.6
     */
    public void setValueOutputFormat(OutputFormat dataFormat);

}
