// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ContinuousQueryFilter.java
//
// Source File Description:
//
// An interface which provides an abstraction to check if an object
// matches the criteria defined in a filter.
package com.ibm.websphere.objectgrid.continuousquery;

import java.io.Serializable;

import com.ibm.websphere.objectgrid.continuousquery.exception.ContinuousQueryException;
import com.ibm.websphere.objectgrid.continuousquery.filter.AbstractCQFilter;
import com.ibm.websphere.objectgrid.continuousquery.filter.FilterContent;

/**
 * An interface which provides an abstraction to check if an object
 * matches the criteria defined in a filter.  Filters are invoked within
 * the eXtreme Scale containers hosting partitions for the map on which the
 * query is defined.
 *
 * All implementations must extend {@link AbstractCQFilter}.
 * 
 * @param <KeyType>  Type of the key object for the map being queried
 * @param <ValueType> Type of the value object for the map being queried
 * @param <AttributeType> Type of the attribute referenced by the attribute path
 * @param <MatchType> Type of the object being compared to
 *
 * @since 8.6
 */
public interface ContinuousQueryFilter<KeyType,ValueType,AttributeType,MatchType> extends Serializable {
    
    /**
     * Checks if the supplied object passes the filter.
     * 
     * @param content 
     *     A representation of the cache entry to be checked
     * @return true if the object matches the filtering criteria, false otherwise
     * @throws ContinuousQueryException
     */
    public boolean filter(FilterContent<KeyType,ValueType> content) throws ContinuousQueryException;

    /**
     * The name used to identify the key objects in POJO maps.
     */
    public String POJO_ADDRESSABLEKEYNAME = "key";
    
    /**
     * The path separator to use when identifying attribute paths for POJO maps.
     */
    public String POJO_PATHSEPARATOR = ".";

}