// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ContinuousQueryListener.java
//
// Source File Description:
//
// See javadoc for class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.continuousquery;

/**
 * This interface should be implemented to get a callback for a continuous query. When a Continuous Query listener is
 * added to a Continuous Query Topic, that listener will be called after every change to the current set of results
 * matching the continuous query.
 * 
 * 
 * @param <KeyType>
 *            Type of the key object for the map being queried
 * @param <ValueType>
 *            Type of the value object for the map being queried
 * 
 * @since 8.6
 */
public interface ContinuousQueryListener<KeyType, ValueType> {

    /**
     * This method will be called by every ContinuousQueryTopic that this listener is added to. The caller will capture
     * any {@link Throwable}s thrown from calls to this method and create an FFDC entry. This
     * {@link ContinuousQueryListener} will not be called again for the same {@link ContinuousQueryNotificationEvent}.
     * Additionally, the performance of this {@link ContinuousQueryListener} may make an impact on the speed with which
     * other {@link ContinuousQueryListener}s are called.
     * 
     * @param event
     *            - {@link ContinuousQueryNotificationEvent} containing information about the change in the query result
     *            set
     * 
     * @see ContinuousQueryTopic#addListener(ContinuousQueryListener)
     * @see ContinuousQueryNotificationEvent
     */
    public void cacheUpdated(ContinuousQueryNotificationEvent<KeyType, ValueType> event);
}
