// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  BinaryLogicalFilter.java
//
// Source File Description:
//
// This is a base class used for binary logic operations.
package com.ibm.websphere.objectgrid.continuousquery.filter;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;

/**
 * This is a base class used for binary logic operations.
 * 
 * @since 8.6
 */
public abstract class BinaryLogicalFilter extends AbstractCQFilter<Object,Object,Object,Object> {
    
    private static final long serialVersionUID = -626899109427011467L;
    ContinuousQueryFilter[] filters;
    
    public BinaryLogicalFilter() { }

    /**
     * Construct a filter which will perform a boolean operation on a set of filters
     * @param filterList the list of filters to be operated on
     */
    public BinaryLogicalFilter(ContinuousQueryFilter... filterList) {
        filters = filterList;
    }

    /**
     * Composes a string representation of the binary logical filter using the operator.
     * 
     * @param operator the operator used to join the filters
     * @return "(<filter> <operator> <filter> <operator> <filter> ...)", 
     *          where <filter> refers to the string representation of {@linkplain AbstractCQFilter},
     *          and <operator> is input parameter.
     */
    protected String createString(String operator) {
        StringBuilder sb = new StringBuilder();
        sb.append("(");
        for (int i = 0; i < filters.length; ++i) {
            sb.append(filters[i].toString());
            if (i != filters.length - 1) sb.append(" " + operator + " ");
        }
        sb.append(")");
        return sb.toString();
    }

}
