// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  IsNotNullFilter.java
//
// Source File Description:
//
// This class checks if an attribute is not null.
package com.ibm.websphere.objectgrid.continuousquery.filter;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;
import com.ibm.websphere.objectgrid.continuousquery.exception.ContinuousQueryException;
import com.ibm.websphere.objectgrid.io.XsDataInputStream;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.DataAttributeInflatable;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.DataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.MapDataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.DataObjectContext;

/**
 * This filter checks if the attribute to be compared is not null.
 * @since 8.6
 */
public class IsNotNullFilter extends AbstractCQFilter<Object,Object,Object,Object> {

    private static final long serialVersionUID = 7811741670034498515L;

    String attributePath;

    /**
     * Constructor for serialization only.
     */
    public IsNotNullFilter() { }

    /**
     * Construct a filter for checking a value from an arbitrary object is not null.
     * 
     * @param attributePath 
     *              @see CompareFilter#CompareFilter(String, Object)
     */
    public IsNotNullFilter(String attributePath) {
        this.attributePath = attributePath;
    }

    /**
     * @see AbstractCQFilter#filter(FilterContent)
     */
    public boolean filter(FilterContent content)  throws ContinuousQueryException {
            Object objectToFilter = getAttribute(content, attributePath);
            if(DataSerializer.SpecialValue.NOT_FOUND.equals(objectToFilter)) return false;
            return objectToFilter != null;
    }

    /**
     * @see AbstractCQFilter#createString()
     * 
     * @return "<attributePath> ISNOTNULL",
     *          where <attributePath> refers to the path for accessing a value from the object to be filtered
     */
    public String createString() {
        return (attributePath == null ? "value" : attributePath) + " ISNOTNULL";
    }

}
