// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  MatchFilter.java
//
// Source File Description:
//
// This class checks if an attribute matches a regular expression pattern. 
package com.ibm.websphere.objectgrid.continuousquery.filter;

import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;
import com.ibm.websphere.objectgrid.continuousquery.exception.ContinuousQueryException;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer;
import com.ibm.ws.ffdc.FFDCFilter;

/**
 * This class checks if an attribute matches a regular expression pattern.
 * @since 8.6
 */
public class MatchFilter extends CompareFilter<Object,Object,Object,String> {

    private static final long serialVersionUID = -982510880144457284L;
    
    /**
     * Constructor for serialization only.
     */
    public MatchFilter() { }

    /**
     * Construct a new regular expression based filter
     * @param attributePath 
     * 		@see CompareFilter#CompareFilter(String, Object)
     * @param pattern 
     * 		@see java.util.regex.Pattern
     */
    public MatchFilter(String attributePath, String pattern) {
        super(attributePath, pattern);
    }

    /**
     * @see ContinuousQueryFilter#filter(FilterContent)
     */
    @Override
    public boolean filter(FilterContent content) throws ContinuousQueryException{
            Object objectToFilter = getAttribute(content, attributePath);
            if(DataSerializer.SpecialValue.NOT_FOUND.equals(objectToFilter)) return false;
            if (objectToFilter != null && objectToCompare != null && objectToCompare instanceof String) {
                try {
                    if (objectToFilter instanceof CharSequence) {
                        return (Pattern.matches((String)objectToCompare, (CharSequence)objectToFilter));
                    } else if (objectToFilter instanceof byte[]) {
                        String vStr = new String((byte[]) objectToFilter);
                        return (Pattern.matches((String)objectToCompare, vStr));
                    } else {
                        String vStr = objectToFilter.toString();
                        return (Pattern.matches((String)objectToCompare, vStr));
                    }
                } catch (PatternSyntaxException e) {
                    FFDCFilter.processException(e, MatchFilter.class.getName() + ".filter", "43", this);
                    return false;
                }
            }
            return false;
    }

    /**
     * @see AbstractCQFilter#createString()
     * 
     * @return "<attributePath> matches <object>",
     *          where <attributePath> refers to the path for accessing a value from the object to be filtered,
     *          and <object> refers to the string representation of the object to be compared with.
     */
    public String createString() {
        return createString("matches");
    }
}
