// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  OrFilter.java
//
// Source File Description:
//
// This class returns true if any filter is true.
package com.ibm.websphere.objectgrid.continuousquery.filter;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;
import com.ibm.websphere.objectgrid.continuousquery.exception.ContinuousQueryException;


/**
 * This filter passes if any child filter is true.
 * @since 8.6  
 */
public class OrFilter extends BinaryLogicalFilter {

    private static final long serialVersionUID = -2612841473501279318L;
    
    public OrFilter() { }

    /**
     * @see BinaryLogicalFilter#BinaryLogicalFilter(ContinuousQueryFilter...)
     */
    public OrFilter(ContinuousQueryFilter... list) {
        super(list);
    }

    /**
     * Will return true if any child filter returns true.  Short circuiting applies
     * so not all child filters are guaranteed to be invoked.
     * @see ContinuousQueryFilter#filter(FilterContent)
     */
    public boolean filter(FilterContent content) throws ContinuousQueryException{
        boolean rc = false;
        for (ContinuousQueryFilter f : filters) {
            rc = rc || f.filter(content);
            if (rc) break;
        }
        return rc;
    }

    /**
     * @see AbstractCQFilter#createString()
     * 
     * @return "(<filter> OR <filter> ...)", 
     *          where <filter> refers to the string representation of {@linkplain AbstractCQFilter}.
     */
    public String createString() {
        return createString("OR");
    }
}
