// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2006, 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  EntityErrorValue.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// bug 4560: DataGrid first drop of working code.
// b10442    XD6.1     11/01/2007   cdjohnson Add partition id and exception
//
package com.ibm.websphere.objectgrid.datagrid;

import com.ibm.websphere.objectgrid.KeyNotFoundException;



/**
 * This is returned as the value when the AgentManager is unable to run the agent on
 * the server, the server is unable to complete the agent's transaction or the agent throws
 * an exception.  This value can be introspected to determine the server and partition
 * in which the error occurred.
 *
 * @since WAS XD 6.1
 */
public interface EntryErrorValue {

	/**
	 * The toString of the exception thrown by the agent.
	 * @return the exception string.
	 */
	String getErrorExceptionString();
	/**
	 * This is the class name for the thrown exception
	 * @return the exception class name
	 * @see KeyNotFoundException
	 */
	String getExceptionClassName();

	/**
	 * The server where this exception occurred
	 * @return the server name
	 */
	String getServerName();

	/**
	 * The exception thrown by the agent.<p>
	 *
	 * This may return a null if the client does not have the required exeption available
	 * on the class path.  Use the <tt>getExceptionClassName</tt> and <tt>getErrorExceptionString</tt>
	 * methods to retrieve details of the exception.
	 * @return the exception
	 * @since WAS XD 6.1 FIX3
	 */
	Throwable getException();

	/**
	 * The partition of the agent that threw the exception.
	 * @return the agent's partition
     * @since WAS XD 6.1 FIX3
	 */
	int getPartition();
}