// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005, 2006, 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ReduceGridAgent.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// b9168        XD6.1     07/19/2007   cdjohnson Fix javadoc.
//
package com.ibm.websphere.objectgrid.datagrid;

import java.io.Serializable;
import java.util.Collection;

import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.Session;

/**
 * A ReduceGridAgent is used to process a set of entries and reduce
 * them to a single result.<p>
 *
 * An agent is submitted using an ObjectMap's {@link AgentManager} and is run on the server ObjectGrid
 * instances that host the data in the map.  The agent is routed to one or more server instances based on
 * the partitions that the optional keys resolve to.
 *
 * If the map is associated with an entity, then the provided key should be an entity
 * instance and the keys returned from processAllEntries map should be
 * an entity with at least all the key attributes populated.  Agent implementations should implement
 * the EntityAgentMixin interface to allow automatic key to entity conversion.<p>
 *
 * ReduceGridAgents are invoked in a separate, independent transaction from the client's transaction.
 * A single client request may result in multiple transactions. This can happen if the ObjectGrid
 * processes a thousand keys using 10 transactions on 10 threads for example, 100 keys per thread.<p>
 *
 * Keys are processed in arbitrary order sequentially or in parallel and may be grouped into one
 * or more transactions.  Agents may run more than once for a key if a subsequent agent call fails
 * in the same transaction, however an agent will only commit a successful process call one time.

 * @see EntityAgentMixin
 * @since WAS XD 6.1
 */
public interface ReduceGridAgent extends Serializable
{
	/**
	 * The method should reduce the entries referenced by the key collection
	 * to a single value. An example would be a method that returns
	 * an average value for an attribute in a collection of objects. Transactions are managed by
	 * the caller. Any exception results in a rollback. If this map is associated with an entity, then a collection
	 * of populated entities are provided to the collection instead Tuple keys. The class used for the entity
	 * is identified using the EntityAgentMixin interface.
     *
     * @param s The Session to use for any work. Transactions are controlled by the caller.
	 * @param map The ObjectMap this agent was invoked for.
	 * @param keys The collection of keys or entities whose values should be reduced.
	 * @return The reduced value calculated from processing the entries specified.
	 */
	Object reduce(Session s, ObjectMap map, Collection/*<Object key>*/ keys);
	/**
	 * This method is called on each partition. The Agent may have extra state including
	 * possible a query to eliminate some partition entries.
     * @param s The Session to use for any work. Transactions are controlled by the caller.
	 * @param map The ObjectMap this agent was invoked for
	 * @return The single value calculated from processing the entries for the partition.
	 */
	Object reduce(Session s, ObjectMap map);

	/**
	 * This is called when further reduction is needed.
	 * @param results the intermediate results to reduce.
	 * @return The single value calculated from processing the intermediate results.
	 */
	Object reduceResults(Collection results);
}
