// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  DeploymentPolicy.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason        Version   Date         Userid    Change Description
// ------------- --------- ------------ --------- -----------------------------------------
// b964          6.1       05/31/2006   vandersn  add deploymentPolicy
// b9388                   10/01/2007   vandersn  additional zone support
// ------------- --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.deployment;

import java.util.Collection;

import com.ibm.websphere.objectgrid.config.ObjectGridConfiguration;


/**
 * Defines a policy for use by the ObjectGrid Placement Service to place partitions across the set of available containers.
 *
 * @since WAS XD 6.1 FIX3
 */
public interface DeploymentPolicy  {

    /**
     * Add an ObjectGridDeployment to this DeploymentPolicy.  
     * 
     * @param objectGridDeployment - the ObjectGridDeployment to add to this DeploymentPolicy.  Must not be null.
     */
    public void addObjectGridDeployment(ObjectGridDeployment objectGridDeployment);

    /**
     * Get an ObjectGridDeployment that has been added to this DeploymentPolicy.  Null is returned if no ObjectGridDeployment
     * is found with a name that matches the objectGridName.<p>
     *
     * The objectGridName is the name of the ObjectGridDeployment's underlying ObjectGridConfiguration object.
     *
     * @param objectGridName the name of the ObjectGridDeployment to retrieve.
     * @return the ObjectGridDeployment object that contains an ObjectGridConfiguration with a name that matches the objectGridName.
     * @see ObjectGridConfiguration
     */
    public ObjectGridDeployment getObjectGridDeployment(String objectGridName);

    /**
     * Get the Collection of ObjectGridDeployment objects that have been added to this DeploymentPolicy.
     *
     * @return a Collection of ObjectGridDeployment objects
     */
    public Collection getObjectGridDeployments();

    /**
     * Remove an ObjectGridDeployment object from this DeploymentPolicy.
     * @param objectGridName the name of the ObjectGridDeployment to remove
     * @return the ObjectGridDeployment that was removed.  Null is returned if no ObjectGridDeployment was found with a matching objectGridName.
     */
    public ObjectGridDeployment removeObjectGridDeployment(String objectGridName);
    
}
