// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ObjectGridDeployment.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason        Version   Date         Userid    Change Description
// ------------- --------- ------------ --------- -----------------------------------------
// b6555         6.1       08/21/2006   vandersn  make public
// ------------- --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.deployment;

import java.util.Collection;

import com.ibm.websphere.objectgrid.config.ObjectGridConfiguration;

/**
 * A component of the DeploymentPolicy that defines how an ObjectGrid should be placed across the set of available containers.
 *
 * Each ObjectGridDeployment must have an underlying ObjectGridConfiguration object.  The ObjectGridConfiguration is necessary for
 * dividing the ObjectGrid's maps into MapSets.
 *
 * @since WAS XD 6.1 FIX3
 */
public interface ObjectGridDeployment {

    /**
     * Get the ObjectGridConfiguration object that will be used to configure the ObjectGrid as it deployed to the
     * containers.  This ObjectGridConfiguration object contains the BackingMapConfiguration objects that are eligible
     * for inclusion in this ObjectGridDeployment's MapSets.
     *
     * @return the ObjectGridConfiguration object
     */
    public ObjectGridConfiguration getObjectGridConfiguration();

    /**
     * Get the name of the ObjectGrid represented by this ObjectGridDeployment.<p>
     * This is the name of the underlying ObjectGridConfiguration object.
     *
     * @return the ObjectGrid name
     */
    public String getObjectGridName();

    /**
     * Add a MapSet to this ObjectGridDeployment.  Each ObjectGridDeployment must have one or more MapSets.  The MapSet must not be null.
     *
     * @param mapSet the MapSet to add to this ObjectGridDeployment
     */
    public void addMapSet(MapSet mapSet);

    /**
     * Get a MapSet that has been added to this ObjectGridDeployment
     *
     * @param mapSetName - the name of the MapSet to retrieve
     * @return the MapSet with the mapSetName specified
     */
    public MapSet getMapSet(String mapSetName);

    /**
     * Get a Collection of MapSets that have been added to this ObjectGridDeployment
     * @return a Collection of MapSet objects
     */
    public Collection getMapSets();

    /**
     * Remove a MapSet from this ObjectGridDeployment.
     * @param mapSetName the name of the MapSet to remove
     * @return the MapSet that was removed or null if no MapSet was found with a name that matches the mapSetName.
     */
    public MapSet removeMapSet(String mapSetName);

}
