// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  FlushModeType.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b1162        XD6.1     07/27/2006   cdjohnson Feature updates for M4
// b1576        6.1       08/11/2006   cdjohnson Feature updates for M5
// bxxxxx       6.1       02/04/2008   cdjohnson Make serializable
// ---------------------------------------------------------------------------------------

package com.ibm.websphere.objectgrid.em;

import java.io.Serializable;

/*
public enum FlushModeType {
    COMMIT,
    AUTO
}
*/


/**
 * Used to specify when the flush behavior of an EntityManager query when running
 * within a transaction.
 *
 * @see com.ibm.websphere.objectgrid.em.Query#setFlushMode(FlushModeType)
 * @see com.ibm.websphere.objectgrid.em.EntityManager#setFlushMode(FlushModeType)
 * @since WAS XD 6.1
 */
public final class FlushModeType implements Serializable {

    private static final long serialVersionUID = 600910643477175914L;

    /**
     * The EntityManager's context is not automatically flushed prior to running
     * the query.<p>
     * Updates to the state of the entities in the current persistent
     * context are not visible to the processing of the query.
     */
    public final static FlushModeType COMMIT = new FlushModeType(0);

    /**
     * The persistent context is automatically flushed prior to running the
     * query.<p>
     * All updates to the state of all entities in the current persistent
     * context which could potentially affect result of a query are made
     * visible to the processing of the query.
     */
    public final static FlushModeType   AUTO = new FlushModeType(1);

    private final static FlushModeType[] PRIVATE_VALUES = new FlushModeType[] { COMMIT, AUTO };

    private int flushMode;
    private FlushModeType(int i) {
        this.flushMode=i;
    }
    /**
     * Hidden default CTOR for enumerated type idiom to use.
     */
    private FlushModeType() {
        this(-1);
    }

    public String toString() {
        switch (flushMode) {
        case 0: return "COMMIT"; //$NON-NLS-1$
        case 1: return "AUTO"; //$NON-NLS-1$
        default: return "UNKNOWN FlushModeType"; //$NON-NLS-1$
        }
    }

    /*
     * Required to avoid creating a different object instance each time the object is deserialized.
     */
    private Object readResolve() {// throws ObjectStreamException {
        return PRIVATE_VALUES[flushMode];
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + flushMode;
        return result;
    }
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final FlushModeType other = (FlushModeType) obj;
        if (flushMode != other.flushMode)
            return false;
        return true;
    }
}