// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006,2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  DynamicServerMBean.java
//
// Source File Description:
//
// Interface for the MBean operations and attributes related to Servers in a dynamic environment
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 8216         XD6101    6-21-2007    joshuad   new code drop
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.management;

import java.net.InetAddress;

import javax.management.openmbean.CompositeData;
import javax.management.openmbean.TabularData;

import com.ibm.websphere.objectgrid.ObjectGridManager;
import com.ibm.websphere.objectgrid.stats.StatsSpec;

/**
 * This MBean interface allows a client process to access different attributes
 * about a specific server process in a dynamic environment. The object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=ObjectGridServer,name=&lt;server&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.<p>
 *
 * <strong>The following notifications are available:</strong><p>
 * <code><strong>og.server.container.coregroup.membership.change</strong></code>
 * <table>
 * <tr><td><strong>Description:</strong><td>All core group membership changes detected by the server's core group manager.</td></tr>
 * <tr><td><strong>UserData:</strong></td><td>The number of members in the core group.</td></tr>
 * <tr><td><strong>Message:</strong></td><td>The name of the core group.</td></tr>
 * <tr><td><strong>Since:</strong></td><td>6.1 FIX 3</td></tr>
 * </table>
 * <p>
 * <code><strong>com.ibm.websphere.objectgrid.log</strong></code>
 * <table>
 * <tr>
 *     <td><strong>Description:</strong></td>
 *     <td>All log messages detected by the log notification filter. See the {@link #setLogNotificationFilter(String)} attribute.</td>
 * </tr>
 * <tr> <td valign="top"><strong>UserData:</strong></td>
 *      <td>A CompositeData with the following items:<p>
 * <table>
 * <tr>
 *     <td><strong><u>Item Name</u></strong></td>
 *     <td><strong><u>Type</u></strong></td>
 *     <td><strong><u>Description</u></strong></td>
 *     <td><strong><u>Since</u></strong></td>
 * </tr>
 * <tr>
 *     <td>LevelName</td>
 *     <td>String</td>
 *     <td>The log record severity level.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>LoggerName</td>
 *     <td>String</td>
 *     <td>The name of the logger.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>SourceClassName</td>
 *     <td>String</td>
 *     <td>The originating log record class name.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>SourceMethodName</td>
 *     <td>String</td>
 *     <td>The originating log record method name.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>SequenceNumber</td>
 *     <td>Long</td>
 *     <td>The log record sequence number</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>ThreadID</td>
 *     <td>Integer</td>
 *     <td>The originating log record thread id.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>ThrownMessage</td>
 *     <td>String</td>
 *     <td>The thrown exception message or empty string, if not available.</td>
 *     <td>8.6</td>
 * </tr>
 * </table>
 * </td></tr>
 * <tr><td><strong>Message:</strong></td><td>The log message.</td></tr>
 * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
 * </table>
 * <p>
 * <code><strong>com.ibm.websphere.objectgrid.ffdc</strong></code>
 * <table>
 * <tr><td><strong>Description:</strong>
 *     <td>All first-failure data captured by the grid server.</td>
 * <tr><td valign="top"><strong>UserData:</strong></td>
 *     <td>A CompositeData with the following items:<p>
 * <table>
 * <tr>
 *     <td><strong><u>Item Name</u></strong></td>
 *     <td><strong><u>Type</u></strong></td>
 *     <td><strong><u>Description</u></strong></td>
 *     <td><strong><u>Since</u></strong></td>
 * </tr>
 * <tr>
 *     <td>SourceID</td>
 *     <td>String</td>
 *     <td>The source code generating the exception.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>ProbeID</td>
 *     <td>String</td>
 *     <td>The source code location associated with the exception captured.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>ExceptionName</td>
 *     <td>String</td>
 *     <td>The exception captured.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>Count</td>
 *     <td>Integer</td>
 *     <td>The current occurrence count for the specified exception.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>DateOfFirstOccurrence</td>
 *     <td>String</td>
 *     <td>The exception's first occurrence time stamp.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>Label</td>
 *     <td>String</td>
 *     <td>The label associated with the captured exception occurrence.</td>
 *     <td>8.6</td>
 * </tr>
 * </table>
 * </td></tr>
 * <tr><td><strong>Message:</strong></td><td>A notification was generated on the server for a new exception.</td></tr>
 * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
 * </table>
 * @since WAS XD 6.1 FIX3, XC10
 */
public interface DynamicServerMBean extends ServerMBean {
    /*
     * Notification for servers joining within the core group.
     */
    
    /**
     * A constant identifying the notification type for all core group membership changes detected by this server.
     * @since 6.1 FIX 3
     */
    public static final String SERVER_COREGROUP_MEMBERSHIP_CHANGE = "og.server.container.coregroup.membership.change";
    
    /**
     * The constant identifying the notification type for log messages.
     * @since 8.6
     */
    public static final String LOG_MESSAGE_NOTIFICATION           = "com.ibm.websphere.objectgrid.log";
    public static final String FFDC_NOTIFICATION                  = "com.ibm.websphere.objectgrid.ffdc";
    public static final String LOG_RECORD_DATA                    = "com.ibm.websphere.objectgrid.management.LogRecordData";
    public static final String THROWN_MESSAGE                     = "ThrownMessage";
    public static final String THREAD_ID                          = "ThreadID";
    public static final String SEQUENCE_NUMBER                    = "SequenceNumber";
    public static final String SOURCE_METHOD_NAME                 = "SourceMethodName";
    public static final String SOURCE_CLASS_NAME                  = "SourceClassName";
    public static final String LOGGER_NAME                        = "LoggerName";
    public static final String LEVEL_NAME                         = "LevelName";
    public static final String FFDC_INCIDENT_DATA                 = "com.ibm.websphere.objectgrid.management.FFDCIncidentData";
    public static final String SOURCE_ID                          = "SourceID";
    public static final String PROBE_ID                           = "ProbeID";
    public static final String LABEL                              = "Label";
    public static final String DATE_OF_FIRST_OCCURRENCE           = "DateOfFirstOccurrence";
    public static final String COUNT                              = "Count";
    public static final String EXCEPTION_NAME                     = "ExceptionName";

    /**
     * Returns the number of available processors for the JVM hosting this server.
     * @see Runtime#availableProcessors()
     * @return The answer from the Runtime call on the JVM hosting this server.
     */
    public int getAvailableProcessors();

    /**
     * Returns the available memory in bytes for the JVM hosting this server.
     * @see Runtime#freeMemory()
     * @return The answer from the Runtime call on the JVM hosting this server.
     */
    public long getFreeMemory();

    /**
     * Returns the maximum memory in bytes for the JVM hosting this server.
     * @see Runtime#maxMemory()
     * @return The answer from the Runtime call on the JVM hosting this server.
     */
    public long getMaxMemory();

    /**
     * Returns the total memory in bytes for the JVM hosting this server.
     * @see Runtime#totalMemory()
     * @return The answer from the Runtime call on the JVM hosting this server.
     */
    public long getTotalMemory();

    /**
     * Returns the host name for this process.
     *
     * @see InetAddress#getHostName()
     * @return The answer from the Runtime call on the JVM hosting this server.
     */
    public String getHostName();

    /**
     * Returns the zone name for this process
     *
     * @return the zone name that was included in the properties used to start the server or
     * DefaultZone if no zone name was used
     */
    public String getZoneName();

    /**
     * Returns true if a replica exists for each primary hosted on this server.
     * Returns false if the server has the only copy of data.
     *
     * @return If server is safe to shutdown.
     */
    public boolean getSafeToShutdown();

    /**
     * Retrieve the current statistics specification for the server.
     * @return a string representation of the statistics specification.
     * @see StatsSpec
     * @since 7.1
     */
    public String getStatsSpec();

    /**
     * Set the statistics specification for the server.
     * @param statsSpec the statistics specification string.
     * @see StatsSpec
     * @since 7.1
     */
    public void setStatsSpec(String statsSpec);

    /**
     * Retrieve the current trace specification for the server.
     * @return the trace specification string.
     * @since 7.1
     */
    public String getTraceSpec();

    /**
     * Set the trace specification for the server.
     * @param traceSpec the statistics specification string.
     * @see ObjectGridManager#setTraceSpecification(String)
     * @since 7.1
     */
    public void setTraceSpec(String traceSpec);

    
    /**
     * Retrieve the environment information for the server (host name, 
     * WebSphere eXtreme Scale version, and additional information).
     * The CompositeData contains the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr> <td>JMXServicePort</td><td>String</td><td>JMX Service Port</td> </tr>
     * <tr> <td>WASServerName</td><td>String</td><td>WebSphere Application Server Full Server Name</td> </tr>
     * <tr> <td>JVMVersion</td><td>String</td><td>JAVA Version</td> </tr>
     * <tr> <td>JMXConnectorPort</td><td>String</td><td>JMX Connector Port</td> </tr>
     * <tr> <td>IPAddress</td><td>String</td><td>IP Address</td> </tr>
     * <tr> <td>JavaVM</td><td>String</td><td>JVM Version</td> </tr>
     * <tr> <td>WASInstallRoot</td><td>String</td><td>WebSphere Application Server Product Directory</td> </tr>
     * <tr> <td>OSGIFrameworkVersion</td><td>String</td><td>OSGi Version</td> </tr>
     * <tr> <td>ClientPort</td><td>String</td><td>Client Port</td> </tr>
     * <tr> <td>HostName</td><td>String</td><td>Host name</td> </tr>
     * <tr> <td>Timestamp</td><td>String</td><td>Time Stamp from Server</td> </tr>
     * <tr> <td>WASBaseVersion</td><td>String</td><td>IBM WebSphere Application Server Version</td> </tr>
     * <tr> <td>OSName</td><td>String</td><td>Operating System</td> </tr>
     * <tr> <td>XSInstallRoot</td><td>String</td><td>WebSphere eXtreme Scale Product Directory</td> </tr>
     * <tr> <td>OSArch</td><td>String</td><td>OS Architecture</td> </tr>
     * <tr> <td>PeerPort</td><td>String</td><td>Peer Port</td> </tr>
     * <tr> <td>ServerType</td><td>String</td><td>Server Type</td> </tr>
     * <tr> <td>HAManagerPort</td><td>String</td><td>HAManager Port</td> </tr>
     * <tr> <td>JVMInstallPath</td><td>String</td><td>JAVA Directory</td> </tr>
     * <tr> <td>XC10Model</td><td>String</td><td>Machine Type and Model</td> </tr>
     * <tr> <td>JavaRuntimeInfo</td><td>String</td><td>JVM Runtime Version</td> </tr>
     * <tr> <td>xioContainerTCPNonSecure</td><td>String</td><td>XIO TCP/IP Port</td> </tr>
     * <tr> <td>JMXServiceURL</td><td>String</td><td>JMX Service Port</td> </tr>
     * <tr> <td>listenerPort</td><td>String</td><td>Listener Port (ORB)</td> </tr>
     * <tr> <td>ServerName</td><td>String</td><td>Server Name</td> </tr>
     * <tr> <td>WASXDVersion</td><td>String</td><td>IBM WebSphere Application Server - XD Version</td> </tr>
     * <tr> <td>WASExpressVersion</td><td>String</td><td>IBM WebSphere Application Server - ND Version</td> </tr>
     * <tr> <td>JavaBitMode</td><td>String</td><td>JAVA Bit Mode</td> </tr>
     * <tr> <td>XSVersion</td><td>String</td><td>WebSphere eXtreme Scale Version</td> </tr>
     * <tr> <td>JVMVendor</td><td>String</td><td>JAVA Vendor</td> </tr>
     * <tr> <td>OSVersion</td><td>String</td><td>Operating System Version</td> </tr>
     * <tr> <td>xioContainerTCPSecure</td><td>String</td><td>XIO TCP/IP SSL Port</td> </tr>
     * <tr> <td>PID</td><td>String</td><td>Process ID</td> </tr>
     * <tr> <td>WASNDVersion</td><td>String</td><td>IBM WebSphere Application Server - ND Version</td> </tr>
     * <tr> <td>ORB Version</td><td>String</td><td>ORB Version</td> </tr>
     * </table>
     * @return CompositeData containing the environment information
     * @since 8.5
     */
    public CompositeData getEnvironmentInfo();

    /**
     * Sets the regular expression filter to be applied in screening log record messages before being broadcasted
     * by the grid server as a JMX Notification of type:
     * <p>
     * <code><strong>com.ibm.websphere.objectgrid.log</strong></code>
     * <table>
     * <tr><td valign="top"><strong>UserData:</strong></td><td>A CompositeData with the following items:<p>
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>LevelName</td>
     *     <td>String</td>
     *     <td>The logging level.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>LoggerName</td>
     *     <td>String</td>
     *     <td>The name of the logger.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SourceClassName</td>
     *     <td>String</td>
     *     <td>The class name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SourceMethodName</td>
     *     <td>String</td>
     *     <td>The method name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SequenceNumber</td>
     *     <td>Long</td>
     *     <td>The sequence number</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ThreadID</td>
     *     <td>Integer</td>
     *     <td>The thread id.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ThrownMessage</td>
     *     <td>String</td>
     *     <td>The thrown exception message or empty string, if not available.</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * </td></tr>
     * <tr><td><strong>Message:</strong></td><td>The log message.</td></tr>
     * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
     * </table>
     * @param regExFilter
     *            The regular expression filter to applied in screening log record messages.
     * @since 8.6
     */
    public void setLogNotificationFilter(String regExFilter);

    /**
     * Retrieves current regular expression filter being applied while screening log record messages
     * before being broadcasted as JMX notifications by the grid server as a JMX Notification of type:
     * <p>
     * <code><strong>com.ibm.websphere.objectgrid.log</strong></code>
     * <table>
     * <tr><td valign="top"><strong>UserData:</strong></td><td>A CompositeData with the following items:<p>
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>LevelName</td>
     *     <td>String</td>
     *     <td>The logging level.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>LoggerName</td>
     *     <td>String</td>
     *     <td>The name of the logger.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SourceClassName</td>
     *     <td>String</td>
     *     <td>The class name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SourceMethodName</td>
     *     <td>String</td>
     *     <td>The method name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SequenceNumber</td>
     *     <td>Long</td>
     *     <td>The sequence number</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ThreadID</td>
     *     <td>Integer</td>
     *     <td>The thread id.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ThrownMessage</td>
     *     <td>String</td>
     *     <td>The thrown exception message or empty string, if not available.</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * </td></tr>
     * <tr><td><strong>Message:</strong></td><td>The log message.</td></tr>
     * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
     * </table>
     * @return  Current regular expression filter being applied in screening log record messages.
     * @since 8.6
     */
    public String getLogNotificationFilter();

    /**
     * Triggers a simulated exception to be captured as a first-failure data capture (FFDC) event (and subsequently
     * broadcasted as a JMX notification), as a means to test and verify the monitoring being enabled on the server. 
     * The JMX notification generated and broadcasted has the following type and content:
     * <p>
     * <code><strong>com.ibm.websphere.objectgrid.ffdc</strong></code>
     * <table>
     * <tr><td><strong>Description:</strong>
     *     <td>All first-failure data captured by the grid server.</td>
     * <tr><td valign="top"><strong>UserData:</strong></td>
     *     <td>A CompositeData with the following items:<p>
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong></td>
     *     <td><strong><u>Type</u></strong></td>
     *     <td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>SourceID</td>
     *     <td>String</td>
     *     <td>The source code generating the exception.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ProbeID</td>
     *     <td>String</td>
     *     <td>The source code location associated with the exception captured.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ExceptionName</td>
     *     <td>String</td>
     *     <td>The exception captured.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>Count</td>
     *     <td>Integer</td>
     *     <td>The current occurrence count for the specified exception.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>DateOfFirstOccurrence</td>
     *     <td>String</td>
     *     <td>The exception's first occurrence time stamp.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>Label</td>
     *     <td>String</td>
     *     <td>The label associated with the captured exception occurrence.</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * </td></tr>
     * <tr><td><strong>Message:</strong></td><td>A simulated first-failure data capture (FFDC) exception notification was generated by the server.</td></tr>
     * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
     * </table>
     * @since 8.6
     */
    public void checkFFDCNotification();

    /**
     * Generates a set of log records with various severity levels, providing a simple means
     * to test and verify when the JMX notification monitoring being enabled on the grid server.
     * Each of the JMX notifications generated and broadcasted has the following type and content:
     * <p>
     * <code><strong>com.ibm.websphere.objectgrid.log</strong></code>
     * <table>
     * <tr><td valign="top"><strong>UserData:</strong></td><td>A CompositeData with the following items:<p>
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>LevelName</td>
     *     <td>String</td>
     *     <td>The logging level.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>LoggerName</td>
     *     <td>String</td>
     *     <td>The name of the logger.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SourceClassName</td>
     *     <td>String</td>
     *     <td>The class name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SourceMethodName</td>
     *     <td>String</td>
     *     <td>The method name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SequenceNumber</td>
     *     <td>Long</td>
     *     <td>The sequence number</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ThreadID</td>
     *     <td>Integer</td>
     *     <td>The thread id.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ThrownMessage</td>
     *     <td>String</td>
     *     <td>A simulated log info/warning/error notification was generated by the server</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * </td></tr>
     * <tr><td><strong>Message:</strong></td><td>A simulated log info/warning/error notification was generated by the server.</td></tr>
     * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
     * </table>
     * @since 8.6
     */
    public void checkLoggingNotification();
    
    /** 
     * Provides revisions for all shards in the server in the form of a TabularData object, 
     * where each CompositeData (row in the TabularData) contains the following items:<p>
     *
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>Grid</td>
     *     <td>String</td>
     *     <td>The object grid name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>MapSet</td>
     *     <td>String</td>
     *     <td>The name of the mapSet.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>PartitionNumber</td>
     *     <td>Integer</td>
     *     <td>The number of a given partition.</td>
     *     <td>8.6</td>
     * </tr>
      * <tr>
     *     <td>ContainerName</td>
     *     <td>String</td>
     *     <td>Name of the container.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ShardType</td>
     *     <td>String</td>
     *     <td>Type of the shard.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>RevisionState</td>
     *     <td>TabularData</td>
     *     <td>The lifetimeId / revisionNumber info for a given partition.</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * 
     * <p>
     * The RevisionState TabularData's CompositeData contains the following items:<p>
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>LifetimeId</td>
     *     <td>Long</td>
     *     <td>The LifetimeId for the shard</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>Revision</td>
     *     <td>Long</td>
     *     <td>The revision number</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * @return A TabularData object with the revision information. 
     * @since 8.6
     */
    public TabularData getRevisions();

    /**
     * Provides revisions for all primary shards in the container in the form of a TabularData object, 
     * where each CompositeData (row in the TabularData) contains the following items:<p>
     * 
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>Grid</td>
     *     <td>String</td>
     *     <td>The object grid name.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>MapSet</td>
     *     <td>String</td>
     *     <td>The name of the mapSet.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>PartitionNumber</td>
     *     <td>Integer</td>
     *     <td>The number of a given partition.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ContainerName</td>
     *     <td>String</td>
     *     <td>Name of the container.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>ShardType</td>
     *     <td>String</td>
     *     <td>Type of the shard.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>RevisionState</td>
     *     <td>TabularData</td>
     *     <td>The lifetimeId / revisionNumber info for a given partition.</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * 
     * <p>
     * The RevisionState TabularData's CompositeData contains the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>LifetimeId</td>
     *     <td>Long</td>
     *     <td>The LifetimeId for the shard</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>Revision</td>
     *     <td>Long</td>
     *     <td>The revision number</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * 
     * @return A TabularData object with the revision information. 
     * @since 8.6
     */
    public TabularData getPrimaryRevisions();
}
