// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/MapEventListener.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:06
//
// Module  :  MapEventListener.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   02/17/2005   sutter    new code drop
// XD60_M2_B.2  WASX.XD   03/24/2005   kjlaw     cleanup javadoc information.
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// XD60_M3_B.25 WASX.XD   04/21/2005   kjlaw     added preloadCompleted event and
//                                               cleanup javadoc.
// 269516       WASX.XD   04/25/2005   sutter    change eviction processing to use standard programming model
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.plugins.io.annotations.PluginOutputFormat;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedEntry;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedValue;

/**
 * This callback interface is implemented by the application when it wants to
 * receive events about a Map such as the eviction of a map entry.
 *
 * @since WAS XD 6.0, XC10
 *
 * @see BackingMap#addMapEventListener(EventListener)
 * @see BackingMap#removeMapEventListener(EventListener)
 * @see EventListener
 */
public interface MapEventListener extends EventListener
{
    /**
     * Invoked when the specified entry is evicted from the map.
     * <p>
     * The eviction could have occurred either by an <code>Evictor</code>'s
     * processing or by invoking one of the <code>invalidate</code> methods on
     * the <code>ObjectMap</code>.
     * <p>
     * For a MapEventListener in an {@link ObjectMap} that is configured to use {@linkplain OutputFormat#RAW}
     * for the keys and values, the keys and values 
     * objects passed will be {@link SerializedKey} or {@link SerializedValue} objects respectively. If required, you
     * can use the {@link SerializedEntry#getObject()} method to retrieve (possibly inflating
     * the serialized object) the original key or value object.<p>
     * 
     * To override the map's output format configuration, use the {@link PluginOutputFormat} 
     * annotation in the implementation class.
     *
     * @param key      The key for the map entry that was evicted.
     * @param value    The value that was in in the map entry evicted. The value object
     *                 should not be modified.
     *
     * @see Evictor
     * @see EvictionEventCallback
     * @see ObjectMap#invalidate(Object, boolean)
     */
    void entryEvicted(Object key, Object value);

    /**
     * Invoked when the preloading of this map has completed.
     * <p>
     * This method is useful to determine when a preload operation finishes
     * if asynchronous preloading is enabled.  In addition if any error occurred
     * during synchronous or asynchronous preload, it is reported with the
     * invocation of this method.
     *
     * @param t        A Throwable object that indicates if preload completed without any
     *                 Throwable occuring during the preload of the map.  A null reference
     *                 indicates preload completed without any Throwable objects occuring
     *                 during the preload of the map.
     *
     * @see Loader#preloadMap(Session, BackingMap)
     * @see BackingMap#setPreloadMode(boolean)
     */
    void preloadCompleted( Throwable t );
}
