// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.7 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/ObjectGridEventListener.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:09
//
// Module  :  ObjectGridEventListener.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M4_B.17 WASX.XD   05/25/2005   sutter    Distributed Event Listener
// 284443       WASX.XD   06/23/2005   sutter    Modify initialize signature to provide a Session
// 294637.1     WASX.XD   08/09/2005   kjlaw     correct javadoc for transactionEnd method.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.util.Collection;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedEntry;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedValue;

/**
 * This interface is used to create an implementation of an event listener for
 * an ObjectGrid.  Instances of <code>ObjectGridEventListener</code>s are set
 * on the <code>ObjectGrid</code> interface.  Any significant events are
 * communicated to the application using the methods outlined below.
 *
 * When using Java 5, this callback also supports new callback annotation mechanism.
 *
 * @since WAS XD 6.0, XC10
 *
 * @see ObjectGrid#addEventListener(EventListener)
 * @see ObjectGrid#removeEventListener(EventListener)
 * @see EventListener
 */
public interface ObjectGridEventListener extends EventListener
{
    /**
     * Invoked when an <code>ObjectGrid</code> is initialized.
     * <p>
     * A usable <code>Session</code> instance is passed to this listener to
     * provide all of the necessary access to the various ObjectGrid objects.
     *
     * @param session a Session instance that this listener is associated with.
     *
     * @see ObjectGrid#initialize()
     */
    void initialize(Session session);

    /*
     * Note:  The activate/suspend life cycle method were removed from this
     * interface since we didn't think "normal" ObjectGridEventListeners would
     * require such methods.  These type of method are really only required
     * for the special Distributed Event Listener.  Thus, that implementation
     * would implement both this Listener interface as well as the StateTransition
     * interface.
     */

    /**
     * Signals the beginning of a <code>Session</code> transaction.
     * <p>
     * A stringified version of the TxID is provided for correlating with the
     * end of the transaction, if so desired.  The type of transaction is also
     * provided by the isWriteThroughEnabled boolean parameter.
     *
     * @param txid Stringified version of the TxID
     * @param isWriteThroughEnabled boolean flag indicating whether the Session
     *                              transaction was started using the
     *                              <code>Session.beginNoWriteThrough()</code>.
     *                              method. <code>false</code> is passed if
     *                              <code>beginNoWriteThrough()</code> was used.
     *
     * @see Session#begin()
     * @see Session#beginNoWriteThrough()
     */
    void transactionBegin(String txid, boolean isWriteThroughEnabled);

    /**
     * <p>Signals the ending of a <code>Session</code> transaction.</p>
     *
     * <p>A string version of the TxID is provided for correlating with the
     * begin of the transaction, if so desired. Map changes are also reported
     * with the collection of <code>LogSequence</code>s passed to this method.
     * Typical uses of this event are for customers doing custom peer
     * invalidation or peer commit push. This event listener gives them the
     * changes. Calls to this method are made after commit and are sequenced so
     * that they are delivered one by one, not in parallel. The event order is
     * the commit and rollback order.</p>
     * <p>
     * For an ObjectGridEventListener receiving changes in an {@link ObjectMap} that is
     * configured to use a {@link OutputFormat#RAW} for the keys or values,
     * the keys and values objects in the LogSequences
     * will be {@link SerializedKey} or {@link SerializedValue} objects respectively. If required, you
     * can use the {@link SerializedEntry#getObject()} method to retrieve (possibly inflating
     * the serialized object) the original key or value object.
     * <p>
     * To override the map's output format configuration, use the {@link PluginOutputFormat} 
     * annotation in the implementation class.
     * 
     * @param txid string version of the TxID
     * @param isWriteThroughEnabled boolean flag indicating whether the Session
     *                              transaction was started using the
     *                              <code>Session.beginNoWriteThrough()</code>.
     *                              method. <code>false</code> is passed if
     *                              <code>beginNoWriteThrough()</code> was used.
     * @param committed a boolean flag indicating whether the transaction was
     *                  committed (<code>true</code>) or rolled back
     *                  (<code>false</code>)
     * @param changes   a Collection of LogSequences representing the changes
     *                  that were committed or rolled back.
     *
     * @see LogSequence#isRollback()
     * @see Session#begin()
     * @see Session#beginNoWriteThrough()
     * @see Session#commit()
     * @see Session#rollback()
     */
    void transactionEnd(String txid, boolean isWriteThroughEnabled, boolean committed, Collection /*<LogSequence>*/ changes);

    /**
     * <p>Called when the <code>ObjectGrid</code> associated with this listener is
     * destroyed.</p>
     *
     * <p>This method is the opposite of the <code>initialize</code> method. When
     * it is called, the listener can free up any resources it uses.</p>
     *
     * @see ObjectGrid#destroy()
     */
    void destroy();
}
