// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.6 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/ObjectTransformer.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:12
//
// Module  :  ObjectTransformer.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M4_B.17 WASX.XD   06/01/2005   sutter    serialization updates
// 281469       WASX.XD   06/07/2005   sutter    New serialize/inflate for versioned values
// 279166       WASX.XD   06/16/2005   sutter    Moved serialize/inflate versioned value methods to OptimisticCallback
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.CopyMode;
import com.ibm.websphere.objectgrid.plugins.io.KeySerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.ValueSerializerPlugin;

/**
 * An ObjectTransformer can be used to serialize map entries that are not defined to
 * be Serializable. This plugin is essential to high performance. Java serialization is
 * a very costly operation. Serialization is the default mechanism for copying objects as
 * well as transmitting objects over the network. Java serialization will normally
 * take around 60-70% of the total CPU cost.
 * <p>
 * If an application provides an ObjectTransformer
 * for each BackingMap, this cost can drop to less than 5%, a significant performance
 * improvement. If a customer has a performance problem, the #1 advice is to provide
 * ObjectTransformers for every Map.
 * <p>
 * An <code>ObjectTransformer</code> implementation that also implements the {@link BackingMapLifecycleListener}
 * interface will be automatically added as an {@link EventListener} on the {@link BackingMap} when the
 * arbiter set on the backing map.
 * <p>
 * An <code>ObjectTransformer</code> may also implement the {@link BackingMapPlugin} interface in order
 * to receive enhanced BackingMap plug-in lifecycle method calls. The plug-in is then
 * also required to correctly implement each of the bean methods related to
 * introspection of its state (for example <code>isInitialized()</code>,
 * <code>isDestroyed()</code>, etc).
 *
 * @since WAS XD 6.0
 *
 * @see BackingMap#addMapEventListener(EventListener)
 * @see BackingMap#setObjectTransformer(ObjectTransformer)
 *
 * @deprecated This plug-in is deprecated in version 7.1.1.  Use the {@link KeySerializerPlugin} or {@link ValueSerializerPlugin} plug-in
 */
@Deprecated
public interface ObjectTransformer
{
    /**
     * Writes a key object to the specified stream.
     * <p>
     * The default implementation simply performs a <code>writeObject</code>
     * invocation passing in the key object.
     *
     * @param key the key object to serialize
     * @param stream the <code>ObjectOutputStream</code> object to write the key object to
     *
     * @throws IOException if an error occurs during processing of this method
     */
    void serializeKey(Object key, ObjectOutputStream stream)
        throws IOException;

    /**
     * Writes a value object to the specified stream.
     * <p>
     * The default implementation simply performs a <code>writeObject</code>
     * invocation passing in the value object.  A proper implementation needs
     * to handle <code>null</code> values if they are supported.
     *
     * @param value the value object to serialize (may be <code>null</code>)
     * @param stream the <code>ObjectOutputStream</code> object to write the value object to
     *
     * @throws IOException if an error occurs during processing of this method
     *
     * @see BackingMap#setNullValuesSupported(boolean)
     */
    void serializeValue(Object value, ObjectOutputStream stream)
        throws IOException;

    /**
     * Reads a key object from the specified stream.
     * <p>
     * The default implementation simply performs a <code>readObject</code>
     * invocation.
     *
     * @param stream the <code>ObjectInputStream</code> object to read the key object from
     *
     * @return de-serialized key object
     *
     * @throws IOException if an error occurs during processing of this method
     * @throws ClassNotFoundException if the key class isn't found
     *
     * @see #serializeKey(Object, ObjectOutputStream)
     */
    Object inflateKey(ObjectInputStream stream)
        throws IOException, ClassNotFoundException;

    /**
     * Reads a value object from the specified stream.
     * <p>
     * The default implementation simply performs a <code>readObject</code>
     * invocation.  A proper implementation needs to handle <code>null</code>
     * values if they are supported.
     *
     * @param stream the <code>ObjectInputStream</code> object to read the value
     *               object from
     *
     * @return de-serialized value object (may be <code>null</code>)
     *
     * @throws IOException if an error occurs during processing of this method
     * @throws ClassNotFoundException if the value class isn't found
     *
     * @see #serializeValue(Object, ObjectOutputStream)
     * @see BackingMap#setNullValuesSupported(boolean)
     */
    Object inflateValue(ObjectInputStream stream)
        throws IOException, ClassNotFoundException;

    /**
     * Creates a copy of a value object.
     * <p>
     * The default implementation attempts to locate and use a defined
     * <code>clone()</code> method on the value object. If the method isn't
     * found, it copies the value object using serialization.  A proper
     * implementation needs to handle <code>null</code> values if they are
     * supported.
     *
     * @param value the value object to copy (may be null)
     *
     * @return a copy of the value.
     *
     * @see CopyMode
     * @see BackingMap#setCopyMode(CopyMode, Class)
     * @see BackingMap#setNullValuesSupported(boolean)
     */
    Object copyValue(Object value);

    /**
     * Creates a copy of a key object.
     * <p>
     * The default implementation attempts to locate and use a defined
     * <code>clone()</code> method on the key object. If the method isn't found,
     * it copies the key object using serialization.
     *
     * @param key the key object to copy
     *
     * @return a copy of the key
     *
     * @see BackingMap#setCopyKey(boolean)
     */
    Object copyKey(Object key);
}
