// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/PartitionableKey.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:21
//
// Module  :  PartitionableKey.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.10 WASX      May 6, 2005  sutter    New code drop
// b3536        XD 6.1    11/28/2006   haowang   range partition support
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

/**
 * This interface is used to identify keys that want to use their
 * own partitioning algorithm.  If a key supports this interface, ObjectGrid
 * will call the <code>ibmGetPartition()</code> method to prime the partitioning 
 * process. The returned Object needs to support the hashCode() method which will be 
 * used by ObjectGrid to spread the data requests across the set of
 * configured partitions.
 * 
 * If this interface is not used, then the hashCode() method on the key
 * itself will be used to prime the partitioning process.
 * 
 * @since WAS XD 6.0.1
 */
public interface PartitionableKey {

    /**
     * Used to obtain an object from the key that should be used for 
     * partitioning.  
     * <p>
     * The returned Object could be a String, an Integer, or any Object that 
     * properly identifies the desired partition.  ObjectGrid will use this 
     * returned Object's <code>hashCode()</code> method to prime the 
     * partitioning process.
     * 
     * @return an Object that identifies the desired partition for this key
     */
    public Object ibmGetPartition();
    
    /**
     * Use to flexibly specify partition id for customers.
     * <p>
     * getCustomPartitionID
     * @return an int partition ID within a range of specified number of partitions 
     */
    //public int getCustomPartitionID();
}
