// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007,2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  WebSphereTransactionCallback.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b9200        6.1 ifix3 07/24/2007   jhanders  New part.
// wi8261       8.5       02/29/2012   cdjohnson deprecated.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.builtins;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.TxID;
import com.ibm.websphere.objectgrid.plugins.TransactionCallback;
import com.ibm.ws.objectgrid.plugins.builtins.UOWManagerSynchronization;
import com.ibm.ws.objectgrid.plugins.builtins.WASTransactionSynchronization;
import com.ibm.ws.objectgrid.runtime.RuntimeInfo;

/**
 * A <tt>TransactionCallback</tt> plugin used to automatically enlist the ObjectGrid transaction
 * as a volatile participant in a WebSphere JTA transaction.  The ObjectGrid transaction is then 
 * called to "commit" or "rollback" its changes immediately after the two-phase commit stage of the JTA 
 * transaction. However, if the ObjectGrid transaction fails to commit or rollback, the failure 
 * does not affect the JTA transaction.<p>
 *
 * The WebSphereTransactionCallback provides an implementation of the <code>isExternalTransactionActive</code>
 * method to determine if a WebSphere Application Server transaction is active. If the method returns true,
 * ObjectGrid automatically begins an ObjectGrid transaction and when the WebSphere transaction
 * commits or rolls back, the ObjectGrid transaction will also be committed or rolled back.<p>
 *
 * This class may be extended to allow for application specific implementations
 * to use the WebSphere transaction manager to control
 * their ObjectGrid and other application resources together.<p>
 *
 * If this class is used outside of a WebSphere Application Server environment, the
 * <code>isExternalTransactionActive</code> will return false.
 *
 * @since WAS XD 6.1 FIX3, XC10
 * @deprecated This plug-in is deprecated in version 8.5.  Use the eXtreme Scale Resource Adapter (wxsra.rar).
 */
public class WebSphereTransactionCallback implements TransactionCallback {
    private final WASTransactionSynchronization tranSynchronization;

    /**
     * The default constructor.
     */
    public WebSphereTransactionCallback() {
        WASTransactionSynchronization tranSync = null;
        RuntimeInfo info = RuntimeInfo.instance();
        if (info.isWASServerProcess()) {
            tranSync = new UOWManagerSynchronization();
        }
        tranSynchronization = tranSync;
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.TransactionCallback#begin(com.ibm.websphere.objectgrid.TxID)
     */
    public void begin(TxID id) {
        // nothing to do
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.TransactionCallback#commit(com.ibm.websphere.objectgrid.TxID)
     */
    public void commit(TxID id) {
        // nothing to do
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.TransactionCallback#initialize(com.ibm.websphere.objectgrid.ObjectGrid)
     */
    public void initialize(ObjectGrid objectGrid) {
        // nothing to do
    }

    /**
     * This method returns <code>true</code> if a WebSphere JTA transaction is active.  If a problem
     * occurs in determining if a transaction is active, an ObjectGridRuntimeException will be thrown.
     *
     * @see com.ibm.websphere.objectgrid.plugins.TransactionCallback#isExternalTransactionActive(com.ibm.websphere.objectgrid.Session)
     * @throws ObjectGridRuntimeException thrown when any exception occurs.  See the cause for the actual exception.
     */
    public boolean isExternalTransactionActive(final Session session) {
        return tranSynchronization == null ? false : tranSynchronization.isWebSphereTransactionActive(session);
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.TransactionCallback#rollback(com.ibm.websphere.objectgrid.TxID)
     */
    public void rollback(TxID id) {
        // nothing to do
    }
}
