// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/index/DynamicIndexCallback.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:22:33
//
// Module : DynamicIndexCallback.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M1_B.5 WASX.XD   09/31/2005   cheng1    Initial Drop
// dxxxxxx      WASX.XD   12/02/2005   cheng1    fix Java doc
// d344949      WXD601.XD 02/08/2006   cheng1    change error method definition
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins.index;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.ObjectGrid;

/**
 * This interface provides a callback mechanism for dynamic indexing process.
 * If applications wish to get notification when a dynamic index is 
 * ready, in error, or destroyed, this callback interface can be implemented 
 * and registered with the dynamic indexing process when creating a dynamic 
 * index.
 * 
 * @since WAS XD 6.0.1
 * 
 * @see BackingMap#createDynamicIndex(MapIndexPlugin, DynamicIndexCallback)
 * @see BackingMap#createDynamicIndex(String, boolean, String, DynamicIndexCallback)
 */
public interface DynamicIndexCallback {

    /**
     * Invoked when the dynamic index is ready.
     * 
     * @param indexName the index name
     */
    public void ready(String indexName);

    /**
     * Invoked when the dynamic indexing process encounters an unexpected error.
     * 
     * @param indexName the index name 
     * @param t a Throwable object that caused the error situation in dynamic 
     *          indexing processing.
     */
    public void error(String indexName, Throwable t);

    /**
     * Invoked when the dynamic index is removed or the BackingMap is destroyed.
     * 
     * @param indexName the index name
     * 
     * @see BackingMap#removeDynamicIndex(String)
     * @see ObjectGrid#destroy()
     */
    public void destroy(String indexName);
}