// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  KeyDataSerializer.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      7/10/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io;

import java.io.IOException;

import com.ibm.websphere.objectgrid.io.XsDataInputStream;
import com.ibm.websphere.objectgrid.io.XsDataOutputStream;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.DataAttributeInflatable;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.Identifiable;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.KeyDataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.DataObjectContext;

/**
 * DataSerializers implement this interface to serialize and inflate key data
 * objects and attributes.<p>
 *
 * Several mix-in interfaces are provided to allow additional capabilities for keys.
 * @since 7.1.1
 */
public interface KeyDataSerializer extends DataSerializer, DataAttributeInflatable, Identifiable {

    /**
     * KeyDataSerializers can implement this optional interface to use alternative partitioning scheme.
     * @since 7.1.1
     */
    public interface Partitionable extends KeyDataSerializer {
        /**
         * Invoked from clients when it needs to use a partition other than the one normally
         * calculated from it's identity.
         * The partition is typically the hashCode of one or more of the key attributes.
         * <p>
         * Do not store or use the data stream beyond the confines of this
         * method call. The data stream is owned by the WebSphere eXtreme Scale 
         * framework and my be closed, pooled or reused as required. 
         * 
         * @param ctx the context of the method call.
         * @param dataObject the data input stream to read the object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}. 
         * Must not be null.
         * @return the partition id to send to the PartitionManager.
         * @throws IOException thrown if there is a problem reading the data.
         */
        public int getPartitionHashCode(DataObjectContext ctx,
                                        XsDataInputStream dataObject) throws IOException;
    }

    /**
     * Retrieve the metadata for the objects that this DataSerializer understands.
     * @return the KeyDataDescriptor metadata, or null if there is no data descriptor.
     */
    public KeyDataDescriptor getKeyDataDescriptor();
}
