package com.ibm.websphere.objectgrid.plugins.io.annotations;

import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;

/**
 * Specifies the class alias that you can use to correlate different class names that 
 * have the same class alias. The class alias and class fields or types are used to identify 
 * a unique class type ID during the object class serialization and de-serialization.<br>
 * (Note: this annotation is currently only used when eXtreme scale data format (XDF) is enabled)<br>
 * 
 * The @ClassAlias annotation can be specified for a user defined class.<br>
 *
 * For example, the syntax for the class could look like :
 * <pre>
 * &#64;ClassAlias("ACME_Employee")
 * </pre>
 * In the above example, ACME_Employee is the class alias for this user defined class.<br>
 * <br>
 * If &#64;ClassAlias annotation is not defined, the name of this class is set as the ClassAlias.<br> 
 * <pre>
 * &#64;ClassAlias("ACME_Employee")
 * class Employee1 {
 *    &#64;FieldAlias("Employee ID")
 *    int empId = -1;
 *
 *    &#64;FieldAlias("Department No.")
 *    int deptId = -1;
 *
 *    &#64;FieldAlias("Year Salary")
 *    float salary = 0;
 *
 *    &#64;FieldAlias("SEX")
 *    String sex = "M";
 *
 *    int age = -1;
 *    String homeAddress = "";
 * </pre>
 * When a ClassAlias and/or FieldAlias are specified in a user defined class, the ClassAlias
 * and/or FieldAlias will be used to create or correlate with an object that are stored or will be stored
 * in the grid. If two user defined classes (in a separate java application environment) have the different
 * class name, but they were marked as the same ClassAlias, and all fields and field types are matched
 * between these 2 classes, they will be correlated with the same class type ID even though they have the
 * different class name. This way will allow the same class metadata to be reused between these 2 classes
 * when running serialization and de-serialization in the different Java application runtime, as well as to
 * shared with C# when the Alias for the class defined in C# and fields are also matched.
 * <br>
 * @since 8.6
 */
@Retention(RUNTIME)
public @interface ClassAlias {
    
    /**
     * The class alias
     * @return the class alias
     */
    public abstract java.lang.String value();
}
