package com.ibm.websphere.objectgrid.plugins.io.annotations;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import com.ibm.websphere.objectgrid.OutputFormat;

/**
 * Identify the data format to be used for plug-ins that accept LogSequnce, LogElement, or
 * cache keys or values.  If not specified, the plug-in will receive cache data in the form
 * specified by the configuration.<p>
 *
 * @see OutputFormat
 * @since 8.6
 */
@Target(TYPE)
@Retention(RUNTIME)
public @interface PluginOutputFormat {

    /**
     * The key OutputFormat to use for all plug-in methods.
     * @return the output format that should be used for all methods that receive cache data keys.
     */
    OutputFormat keyFormat() default OutputFormat.UNDEFINED;

    /**
     * The key OutputFormat to use for all plug-in methods.
     * @return the output format that should be used for all methods that receive cache data keys.
     */
    OutputFormat valueFormat() default OutputFormat.UNDEFINED;
}
