// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  AttributeType.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io.datadescriptor;

/**
 * Identifies all types that can be set on an {@link Attribute}.
 * The attribute types are used to identify how the eXtreme Scale runtime can
 * access and store data appropriately.
 * @since 7.1.1
 * @see Attribute
 */
public enum AttributeType {

    /**
     * A binary value representing true or false.<p>
     * The equivalent Java data type is: boolean or java.lang.Boolean
     */
    BOOLEAN,

    /**
     * A 8-bit integer.
     * The equivalent Java data type is: byte or java.lang.Byte
     */
    BYTE,

    /**
     * Represents an array of bytes.
     * The equivalent Java data type is: byte[]
     */
    BYTES,

    /**
     * A 16-bit integer.
     * The equivalent Java data type is: short or java.lang.Short
     */
    INT16,

    /**
     * A 32-bit integer.
     * The equivalent Java data type is: int or java.lang.Integer
     */
    INT32,

    /**
     * A 64-bit integer.
     * The equivalent Java data type is: long or java.lang.Long
     */
    INT64,

    /**
     * An integer with arbitrary precision.
     * The equivalent Java data type is: java.math.BigInteger
     */
    BIGINT,

    /**
     * Represents a single floating point decimal.
     * The equivalent Java data type is: float or java.lang.Double
     */
    FLOAT,

    /**
     * Represents a double floating point decimal.
     * The equivalent Java data type is: float or java.lang.Double
     */
    DOUBLE,

    /**
     * An arbitrary-precision signed decimal.
     * The equivalent Java data type is: java.math.BigDecimal
     */
    BIGDECIMAL,

    /**
     * Represents a single unicode character.
     * The equivalent Java data type is: char or java.lang.Character
     */
    CHAR,

    /**
     * Represents a date.
     * The equivalent Java data type is: java.sql.Date
     */
    DATE,

    /**
     * Represents a time.
     * The equivalent Java data is: java.sql.Time
     */
    TIME,

    /**
     * Represents a date and time.
     * The equivalent Java data type is: java.util.Date
     */
    DATETIME,

    /**
     * Represents a date and time including nanos.
     * The equivalent Java data type is: java.sql.Timestamp
     */
    TIMESTAMP,

    /**
     * Represents a double floating point decimal.
     * The equivalent Java data type is: float or java.lang.Double
     */
    EMBEDDED,

    /**
     * Represents a variable length unicode string.
     * The equivalent Java data type is: java.lang.String
     */
    STRING,

    /**
     * Represents an enumeration.
     * The equivalent Java data type is: enum
     */
    ENUM,

    /**
     * Represents an arbitrary serializable Object.
     * The equivalent Java data type is: java.lang.Object
     */
    OBJECT
}
