// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Source File Description: See javadoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- ------------------------------------------
// bug 18270     7.1       03/22/2 010  jpcammar  Creation.
// ------------ --------- ------------ --------- ------------------------------------------

package com.ibm.websphere.objectgrid.rest;

import com.ibm.websphere.objectgrid.ObjectGridManager;

/**
 *
 * Java MBean interface for the WebSphere eXtreme Scale REST data service.
 *
 * Provides operations to dynamically interact with the WebSphere eXtreme Scale
 * REST Data Service dynamically through JMX.
 *
 * @since 7.1
 *
 */
public interface RestServiceMBean {

    /**
     * Enables or disables trace for the REST data service.
     *
     * @param enableTrace
     *            A value of "true" enables debug tracing. A value of "false"
     *            disables debug tracing.
     *
     * @throws IllegalArgumentException
     *             If argument does not equal value of "true" or "false".
     *
     * @see ObjectGridManager#setTraceEnabled(boolean)
     */
    public void setTraceEnabled(String enableTrace);

    /**
     * Set the trace specification for the JVM in which the REST data service is running.
     * <p>
     * This operation is a replace operation, not an append operation. The specification should be of the form:
     * <p>
     * <pre> TraceString := &lt;ComponentString&gt;(:&lt;ComponentString&gt;)* ComponentString :=  &lt;ComponentName&gt;=&lt;type>=&lt;state&gt;(,&lt;type&gt;=&lt;state&gt;)*
     * ComponentName := a java String state := [enabled|disabled] type := [all|debug|event|entryExit] </pre>
     * <p>
     * For example, ObjectGridRest*=all=enabled
     *
     * @param traceString
     *            A valid trace specification string such as
     *            ObjectGridRest*=all=enabled.
     *
     * @throws IllegalArgumentException
     *             If the argument is null or a zero-length string.
     *
     * @see ObjectGridManager#setTraceSpecification(String)
     */
    public void setTraceSpec(String traceString);

}
