// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/security/config/SSLConfiguration.java, WAS.objectgrid, WASX.XD, b0614.107 2/24/06 13:00:58
//
// Module  :  SSLConfiguration.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.13 WASX.XD   04/01/2005   jitang    DataGrid - JAAS / Security - Drop 1 - Basic Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.config;

import java.io.Serializable;

/**
 * <p>This interface represents an SSL Configuration. Users can call
 * ClientSecurityConfigurationFactory.getSSLConfiguration() to get 
 * an instance of SSLConfiguration.</p>
 * 
 * <p>An SSL configuration contains the following properties:
 * <ul>
 * <li>alias: the alias in the key store to represent the client</li>
 * <li>protocol: the SSL protocol</li>
 * <li>certReqSubjectDN: the required Subject DN in the peer certificate</li>
 * <li>contextProvider: the JSSE context provider</li>
 * <li>keyStore: the key store file name</li>
 * <li>keyStoreType: the key store file format</li>
 * <li>keyStorePassword: the password to protect the key store file </li>
 * <li>trustStore: the trust store file name</li>
 * <li>trustStoreType: the trust store file format</li>
 * <li>trustStorePassword: the password to protect the trust store file </li>
 * </ul>
 * 
 * <p>Users are not recommended to implement this interface in case more methods
 * are added in future releases.
 * 
 * @since WAS XD 6.0.1, XC10
 * 
 * @see ClientSecurityConfiguration#setSSLConfiguration(SSLConfiguration)
 */

public interface SSLConfiguration extends Serializable {

    /**
     * Gets the SSL alias name for this SSL configuration.
     * 
     * @return   the argument that was passed to the <code>setAlias(String)</code> 
     *           method of this interface or <code>null</code> if 
     *           <code>setAlias</code> was not previously called for this object.
     * 
     * @see #setAlias(String)  
     */
    String getAlias();

    /** 
     * Sets the SSL alias name for this SSL configuration.
     * 
     * @param alias the SSL alias name
     */
    void setAlias(String alias);

    /**
     * Gets the SSL protocol for this SSL configuration. 
     * 
     * @return   the argument that was passed to the <code>setProtocol(String)</code> 
     *           method of this interface or <code>null</code> if
     *           <code>setProtocol</code> was not previously called for this object.
     * 
     * @see #setProtocol(String)
     */
    String getProtocol();

    /**
     * Sets the SSL protocol for this SSL configuration.
     * <p> 
     * An SSL protocol can be SSL, TLS, SSLv2, SSLv3, etc., depending on which 
     * JDK is being used. 
     * 
     * @param protocol the SSL protocol
     */
    void setProtocol(String protocol);

    /**
     * Gets the JSSE context provider for this SSL configuration.
     *   
     * @return   the argument that was passed to the 
     *           <code>setContextProvider(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setContextProvider</code> was not  
     *           previously called for this object.
     * 
     * @see #setContextProvider(String)
     */
    String getContextProvider();

    /**
     * Sets the JSSE context provider for this SSL configuration. 
     * <p>
     * A JSSE context provider can be SunJSSE, IBMJSSE2, IBMJSSE, etc., depending 
     * on which JDK is being used.
     * 
     * @param contextProvider the SSL context provider
     */
    void setContextProvider(String contextProvider);

    /**
     * Gets the key store file path name for this SSL configuration. 
     * 
     * @return   the absolute path of the argument that was passed to the 
     *           <code>setKeyStore(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setKeyStore</code> was not  
     *           previously called for this object.
     * 
     * @see #setKeyStore(String)
     */
    String getKeyStore();

    /**
     * Sets the key store file path name for this SSL configuration.  
     * <p> 
     * A keystore maintains the private key of an entity, as well as its 
     * corresponding public key certificates.
     * 
     * @param keyStore the file path name of a keystore file
     * 
     * @throws IllegalArgumentException if keyStore is <code>null</code>
     *         or a zero length String
     * 
     * @see #setKeyStorePassword(String)
     * @see #setKeyStoreType(String)
     */
    void setKeyStore(String keyStore);

    /**
     * Gets the password of the key store for this SSL configuration. 
     * 
     * @return   the argument that was passed to the 
     *           <code>setKeyStorePassword(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setKeyStorePassword</code> was not  
     *           previously called for this object.
     * 
     * @see #setKeyStorePassword(String)
     */
    String getKeyStorePassword();

    /**
     * Sets the password of the key store for this SSL configuration.
     *  
     * @param keyStorePassword the password to access the keystore
     * 
     * @see #setKeyStore(String)
     * @see #setKeyStoreType(String)
     */
    void setKeyStorePassword(String keyStorePassword);

    /**
     * Gets the type of the key store for this SSL configuration. 
     * 
     * @return   the argument that was passed to the 
     *           <code>setKeyStoreType(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setKeyStoreType</code> was not  
     *           previously called for this object.
     * 
     * @see #setKeyStoreType(String)
     */
    String getKeyStoreType();

    /**
     * Sets the type of key store for this SSL configuration. 
     * <p>
     * A key store type can be JKS, JCEK, etc.
     * 
     * @param keyStoreType the type of the keystore
     * 
     * @see #setKeyStore(String)
     * @see #setKeyStorePassword(String)
     */
    void setKeyStoreType(String keyStoreType);

    /**
     * Gets the trust store file path name for this SSL configuration.
     * 
     * @return   the absolute path of the argument that was passed to the 
     *           <code>setTrustStore(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setTrustStore</code> was not  
     *           previously called for this object.
     * 
     * @see #setTrustStore(String)
     */
    String getTrustStore();

    /**
     * Sets the trust store file path name for this SSL configuration. 
     * <p>
     * A truststore contains certificates for the signers that are trusted in 
     * the environment where the truststore is used. 
     *  
     * @param trustStore the file path name to a trust store
     * 
     * @throws IllegalArgumentException if trustStore is <code>null</code>
     *         or a zero length String
     * 
     * @see #setTrustStorePassword(String)
     * @see #setTrustStoreType(String)
     */
    void setTrustStore(String trustStore);

    /**
     * Gets the password of the trust store for this SSL configuration. 
     * 
     * @return   the argument that was passed to the 
     *           <code>setTrustStorePassword(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setTrustStorePassword</code> was not  
     *           previously called for this object.
     * 
     * @see #setTrustStorePassword(String)
     */
    String getTrustStorePassword();

    /**
     * Sets the password of the trust store for this SSL configuration.
     * 
     * @param trustStorePassword the password to access the trust store
     * 
     * @see #setTrustStore(String)
     * @see #setTrustStoreType(String)
     */
    void setTrustStorePassword(String trustStorePassword);

    /**
     * Gets the type of the trust store for this SSL configuration.
     * 
     * @return   the argument that was passed to the 
     *           <code>setTrustStoreType(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setTrustStoreType</code> was not  
     *           previously called for this object.
     * 
     * @see #setTrustStoreType(String)
     */
    String getTrustStoreType();

    /**
     * Sets the type of the trust store for this SSL configuration.
     * <p>
     * A trust store type can be JKS, JCEK, etc.
     * 
     * @param trustStoreType the type of the trust store
     * 
     * @see #setTrustStore(String)
     * @see #setTrustStorePassword(String)
     */
    void setTrustStoreType(String trustStoreType);

    /**
     * Gets the required subject DN in the peer certificate for this SSL configuration. 
     * 
     * @return   the argument that was passed to the 
     *           <code>setCertReqSubjectDN(String)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setCertReqSubjectDN</code> was not  
     *           previously called for this object.
     */
    String getCertReqSubjectDN();

    /**
     * Sets the required subject DN in the peer certificate for this SSL configuration.
     * <p>
     * A connection will be opened only if the peer certficate's subject DN 
     * matches this value. Otherwise, the connection will be closed.
     * 
     * @param subjectDN the required subject DN in the peer certificate
     */
    void setCertReqSubjectDN(String subjectDN);

    /**
     * Checks two SSLConfiguration objects for equality.
     *
     * @param config the SSLConfiguration object we are testing for equality 
     *        with this object.
     *
     * @return <code>true</code> if both SSLConfiguration objects are equivalent.
     */
    boolean equals(SSLConfiguration config);
}

