// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.2 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/security/plugins/CredentialGenerator.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:24:30
//
// Module  :  CredentialGenerator.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M3_Bx  WASX.XD   11/15/2005   jitang    ObjectGrid security support
// PK17543      WASX.XD   01/11/2006   jitang    Update JavaDoc
// d344266      WXD601.XD 02/08/2006   jitang    Add setProperties method
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.plugins;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.security.config.ClientSecurityConfiguration;

/**
 * <p>This plugin is used to get a Credential representing this client. It is a 
 * factory for the <code>Credential</code> object. 
 * 
 * <p>One example implementation is to return a <code>Credential</code> object 
 * containing a user ID and password pair. The implementation of the Credential 
 * generated by an implementation of this class must to be understood by the 
 * server's <code>Authenticator</code> plugin. 
 * 
 * <p>An implementation class of this interface should have a default constructor.
 * Users can set the implementation 
 * class name (credentialGeneratorClass) in the client security configuration property 
 * file. The client runtime constructs an object of this implementation class 
 * and calls getCredential() to get the <code>Credential</code> to connect to 
 * an ObjectGrid cluster.   
 * 
 * <p>Users can also specify the addtional properties for this factory using the 
 * credentialGeneratorProps property in the client security configuration property file. 
 * These properties will be passed to this 
 * factory by calling the <code>setProperties(String)</code> method. This way, 
 * you can customize the CredentialGenerator factory.
 * 
 * <p>Users can also set CredentialGenerator programmatically by calling <code>
 * ClientSecurityCinfiguration.setCredentialGenerator(CredentialGenerator) method.
 * 
 * <p>For example, you can have the following settings in the client security configuration 
 * property file: 
 * <code>
 * <ul>
 * <li>credentialGeneratorClass=com.myco.CredGenFactory
 * <li>credentialGeneratorProps=user1 password1
 * </ul>
 * </code>
 * A String "user1 password1" is passed to the <code>setProperties(String)</code> 
 * method, with the "user1" indicating the user name, and "password1" indicating the password.
 *   
 * <p>ObjectGrid provides two built-in implementations for this interface:
 * <ul> 
 * <li>com.ibm.websphere.objectgrid.security.plugins.builtins.UserPasswordCredentialGenerator:
 * A credential generator generating a UserPasswordCredential containing a user ID and password pair. </li>
 * <li>com.ibm.websphere.objectgrid.security.plugins.builtins.WSTokenCredentialGenerator:
 * A credential generator generating a WSTokenCredential containing WebSphere Application Server 
 * specific authentication and authorization tokens. </li>
 * </ul>
 * 
 * <p>The relationship between CredentialGenerator and Credential can be one to one 
 * relationship or one to many relationship. For example. the UserPasswordCredentialGenerator 
 * has a one to one relationship with UserPasswordCredential, but the WSTokenCredentialGenerator
 * has a one to many relationship with WSTokenCredential because it could generate
 * different WSTokenCredential based on what Subject is associated with the current 
 * thread.
 * 
 * <P>Refer to the respective JavaDoc for more details.</p>
 * 
 * @since WAS XD 6.0.1, XC10
 * 
 * @see Authenticator
 * @see ClientSecurityConfiguration#setCredentialGenerator(CredentialGenerator)
 * @see Credential
 * @see ObjectGrid#getSession(CredentialGenerator)
 */
public interface CredentialGenerator {

    /**
     * Gets a <code>Credential</code> which represents the client.
     * 
     * @return the Credential representing the client
     * 
     * @throws CannotGenerateCredentialException if a failure occurs when 
     *         generating the Credential for the client.
     * 
     * @see Credential
     */
    Credential getCredential() throws CannotGenerateCredentialException;

    /**
     * <p>Set the user defined properties to the factory. 
     * 
     * <p>This method is used to set CredentialGenerator properties 
     * to the object. These properties can be set using the credentialGeneratorProps 
     * property in the client security configuration property file.
     * This way, you can customize your factory.
     * 
     * @param properties user defined properties
     */
    void setProperties(String properties);
}