// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/security/plugins/SecureTokenManager.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:24:49
//
// Module  :  SecureTokenManager.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d329154      WASX.XD   12/05/2005   jitang    Open up SecureToken interface.
// PK17543      WASX.XD   01/11/2006   jitang    Update JavaDoc
// d344266      WXD601.XD 02/08/2006   jitang    Add setProperties method
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.plugins;

import com.ibm.websphere.objectgrid.security.ObjectGridSecurityException;
import com.ibm.websphere.objectgrid.security.SecurityConstants;

/**
 * <p>This interface is used by ObjectGrid servers to transform an object to a 
 * secure token and vice versa. A secure token is a byte array. 
 * 
 * <p>Here is one example of a possible usage: When a server joins the cluster, 
 * the joining server needs to present a password to the president server in the 
 * cluster. Before sending the password out, the joining server calls the 
 * <code>generateToken(Object)</code> method to generate a token for this 
 * password. The token should be hard to break so the password can be protected 
 * securely. The token will then be sent across the wire. Usually the token is 
 * associated with a time stamp so the malicious replay attack will be difficult.
 * On the receving side, the server calls the <code>verifyToken(byte[])</code> 
 * method to verfiy the token and reconstruct the corresponding object from the 
 * token. 
 * 
 * <P>ObjectGrid utilizes JCE to provide a default implementation of this 
 * interface. In this implementation, when generating the token, the object is 
 * encrypted with a time stamp and then signed.  To verify a token, the token's 
 * signature is verified and then decrypted. This implementation will need a key
 * store configured in the ObjectGrid servers to support the data 
 * encrypting and decrypting and signature signing and verifying. Please use 
 * security.ogserver.props for the secure token key settings.
 * 
 * <p>An implementation class should have a default constructor. Users can set the 
 * CustomSecureTokenManagerProps property in the server security configuration property 
 * file. This property will be set on the object using the setProperties(String) method.
 * 
 * @since WAS XD 6.0.1
 * 
 * @see SecurityConstants#SECURE_TOKEN_MANAGER_CUSTOM_STRING
 * @see SecurityConstants#SECURE_TOKEN_MANAGER_DEFAULT_STRING 
 */
public interface SecureTokenManager {

    /**
     * Generates the token for the specified object. 
     * <p>
     * The generated token should be hard to break. 
     * 
     * @param o the object to be protected
     * 
     * @return a token representing the object to be protected
     * 
     * @throws ObjectGridSecurityException if any exception occurs during 
     *         generation of the token byte array 
     */
    byte[] generateToken(Object o) throws ObjectGridSecurityException;

    /**
     * Verifies the token and reconstruct the object.
     * 
     * @param bytes the token byte array representing the protected object.
     * 
     * @return the protected object
     *  
     * @throws ObjectGridSecurityException if any exception occurs during 
     *         verification of the token byte array 
     */
    Object verifyToken(byte[] bytes) throws ObjectGridSecurityException;

    /**
     * Sets the user defined properties to the factory.
     * <p>
     * This method is used to set SecureTokenManager properties 
     * to the object. These properties can be set using the "SecureTokenManagerProps" 
     * property in the server security configuration property file.
     * This way, you can customize your factory.
     * 
     * @param properties user defined properties
     */
    void setProperties(String properties);
}