// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.2 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/security/plugins/SubjectSource.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:24:53
//
// Module  :  SubjectSource.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M4_B.19 WASX.XD   06/02/2005   jitang    JAAS / Security Drop 2 - Dist. J2SE/XD integration
//
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.plugins;

import javax.security.auth.Subject;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.security.ObjectGridSecurityException;

/**
 * <p>This plugin can be used to get a <code>Subject</code> which represents 
 * an ObjectGrid client. The <code>Subject</code> will then be used for 
 * ObjectGrid authorization. The <code>getSubject()</code> method is called by the
 * ObjectGrid runtime when the <code>ObjectGrid.getSession()</code> method is 
 * used to get a <code>Session</code> and ObjectGrid security is enabled.</p>
 * 
 * <p>This plug-in is useful for an already authenticated client: it can retrieve
 * the authenticated Subject object and then pass it to the ObjectGrid runtime. 
 * Therefore, there is no need for another authentication.</p>
 * 
 * <p>For example, users can use 
 * <pre>
 * Subject.getSubject(AccessControlContext)
 * </pre>
 * to get the subject associated with the AccessControlContext and return it in the 
 * <code>getSubject()</code> implementation. 
 * 
 * @since WAS XD 6.0
 * 
 * @see ObjectGrid#getSession()
 * @see ObjectGrid#setSubjectSource(SubjectSource)
 */
public interface SubjectSource {

    /**
     * Gets a <code>Subject</code> representing the ObjectGrid client.
     * 
     * @return a Subject object
     * 
     * @throws ObjectGridSecurityException if any exception occurs during 
     *         Subject retrieval
     */
    Subject getSubject() throws ObjectGridSecurityException;
}