// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.5 XD/ws/code/objectgrid.impl/src/com/ibm/websphere/objectgrid/security/plugins/builtins/CertificateMappingAuthenticator.java, WAS.objectgrid, WASX.XD, b0614.107 3/16/06 17:40:11
//
// Module  :  CertificateMappingAuthenticator.java
//
// Source File Description:
//
// Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.13 WASX.XD   04/05/2005   jitang    ObjectGrid - JAAS / Security - Drop 1 - Basic Support
//
// ------------ ---------- ------------ -------------------------------------------------
package com.ibm.websphere.objectgrid.security.plugins.builtins;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Arrays;

import javax.security.auth.Subject;
import javax.security.auth.x500.X500Principal;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.security.plugins.Authenticator;
import com.ibm.websphere.objectgrid.security.plugins.Credential;
import com.ibm.websphere.objectgrid.security.plugins.ExpiredCredentialException;
import com.ibm.websphere.objectgrid.security.plugins.InvalidCredentialException;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.util.ObjectGridUtil;

/**
 * This class is an implementation for the <code>Authenticator</code> interface
 * when client certificate authentication is used.
 * <p>
 * This implementation will extract the base distinguished name (DN) the
 * certificate and create a X500Principal with the DN. A Subject object will
 * also be created with this X500Principal object.
 * <p>
 * This class is only for sample and quick testing purpose. Users should
 * write your own Authenticator implementation which can fit better into
 * the environment.
 *
 * @since WAS XD 6.0.1
 *
 * @see Authenticator
 * @see ClientCertificateCredential
 * @deprecated This plug-in is deprecated in version 8.6.  Use the {@link LDAPAuthenticator}, the {@link KeyStoreLoginAuthenticator}, or a custom {@link Authenticator}.
 */
public class CertificateMappingAuthenticator implements Authenticator {

    private static final String CLASS_NAME = CertificateMappingAuthenticator.class.getName();

    private static final TraceComponent TC = Tr.register(
            CLASS_NAME,
            Constants.TR_SECURITY_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    /**
     *  Creates a new CertificateMappingAuthenicator.
     */
    public CertificateMappingAuthenticator() {
        super();
    }

    /**
     * Authenticates a <code>ClientCertificateCredential</code>.
     * <p>
     * Extracts the distinguished name (DN) associated with the Subject
     * field in the certificate and then creates a X500Principal object.
     * A Subject object is created with this principal object.
     *
     * @param credential the credential to authenticate
     *
     * @throws InvalidCredentialException if credential isn't a
     *         ClientCertificateCredential or some error occurs during processing
     *         of the supplied ClientCertificateCredential
     *
     * @throws ExpiredCredentialException if credential is expired.  This exception
     *         is not used by this implementation
     *
     * @see Authenticator#authenticate(Credential)
     */
    public Subject authenticate(Credential credential) throws InvalidCredentialException, ExpiredCredentialException {
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled()) {
            Tr.entry(TC, "authenticate", ObjectGridUtil
                    .populateArrayForTrace(new Object[] { this, credential }));
        }

        if (credential == null) {
            if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
                Tr.exit(
                    TC,
                    "authenticate",
                    "InvalidCredentialException(null credential)");
            throw new InvalidCredentialException("Supplied credential is null");
        }

        if (! (credential instanceof ClientCertificateCredential) ) {
            if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
                Tr.exit(
                    TC,
                    "authenticate",
                    "InvalidCredentialException(wrong credential type)");
            throw new InvalidCredentialException("Supplied credential is not a ClientCertificateCredential");
        }

        Certificate[] certs = ((ClientCertificateCredential) credential).getCerts();

        final Subject subject = new Subject();

        if (certs != null && certs.length > 0) {

            // If the first certificate is an X509Certificate
            if (certs[0] instanceof X509Certificate) {
                try {
                    // Get the first certificate which represents the user
                    X509Certificate certX509 = (X509Certificate) certs[0];

                    // Create a principal
                    final X500Principal principal = new X500Principal(certX509
                            .getIssuerDN()
                            .getName());

                    AccessController.doPrivileged(new PrivilegedAction() {
                        public Object run() {
                            subject.getPrincipals().add(principal);
                            return null;
                        }
                    });

                    // Create the certification path object and add it to the
                    // public credential set
                    CertificateFactory factory = CertificateFactory.getInstance("X.509");
                    final java.security.cert.CertPath certPath = factory.generateCertPath(Arrays
                            .asList(certs));

                    AccessController.doPrivileged(new PrivilegedAction() {
                        public Object run() {
                            subject.getPublicCredentials().add(certPath);
                            return null;
                        }
                    });

                    // TODO Do we need private key here?

                }
                catch (CertificateException ce) {
                    FFDCFilter.processException(ce, CLASS_NAME + ".authenticate", "120", this);
                    if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
                        Tr.exit(TC, "authenticate", "CertificateException");
                    throw new InvalidCredentialException(ce);
                }
            }
            else {
                if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
                    Tr.exit(
                        TC,
                        "authenticate",
                        "InvalidCredentialException");
                throw new InvalidCredentialException(
                    "The client certificate is not valid. The first certificate on the chain is expcted to be a X509Certificate.");
            }
        }
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
            Tr.exit(TC, "authenticate", subject);
        return subject;
    }
}

