// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid.client/src/com/ibm/websphere/objectgrid/security/plugins/builtins/ClientCertificateCredential.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 18:21:59
//
// Module  :  ClientCertificateCredential.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M3_Bx  WASX.XD   11/15/2005   jitang    ObjectGrid security support
// PK17543      WASX.XD   01/11/2006   jitang    Update JavaDoc
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.plugins.builtins;

import java.security.cert.Certificate;

import com.ibm.websphere.objectgrid.security.plugins.Credential;

/**
 * This class represents a credential which contains a SSL client certificate 
 * chain. This credential can be used for the ObjectGrid client certificate 
 * authentication.
 * 
 * @since WAS XD 6.0.1
 * 
 * @see Credential
 */
public class ClientCertificateCredential implements Credential {

    private static final long serialVersionUID = -8684006735219650046L;

    private Certificate[] ivCerts = null;

    /**
     * Creates a new ClientCertificateCredential using the specified
     * certificates.
     * 
     * @param certs the client certificate chain array
     */
    public ClientCertificateCredential(Certificate[] certs) {
        super();
        this.ivCerts = certs;
    }

    /**
     * Gets the client certificate chain array.
     * 
     * @return   the argument that was passed to the constructor 
     *           or the <code>setCerts(Certificate[])</code> 
     *           method of this class 
     * 
     * @see #setCerts(Certificate[])
     */
    public Certificate[] getCerts() {
        return ivCerts;
    }

    /**
     * Sets the client certificate chain array.
     * 
     * @param certs a client certification chain array
     */
    public void setCerts(Certificate[] certs) {
        this.ivCerts = certs;
    }

    /**
     * Checks two ClientCertificateCredential objects for equality.
     * <p>
     * Two ClientCertficateCredential are equal if and only their client certificate
     * chains are equal. That is, the same-index elements of their certificate chain array
     * are equal.
     * 
     * @param o the object we are testing for equality with this object.
     *
     * @return <code>true</code> if both ClientCertificateCredential objects are equivalent.
     * 
     * @see Credential#equals(Object)
     */
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (o instanceof ClientCertificateCredential) {
            ClientCertificateCredential other = (ClientCertificateCredential) o;
            if (ivCerts == null) {
                return other.ivCerts == null;
            }
            if (other.ivCerts.length != ivCerts.length) {
                return false;
            }
            for (int i=0; i<ivCerts.length; i++) {
                if (!ivCerts[i].equals(other.ivCerts[i])) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    /**
     * Returns the hashcode of the ClientCertificateCredential object.
     * 
     * @return the hash code of this object
     * 
     * @see Credential#hashCode()
     */
    public int hashCode() {
        return ivCerts.hashCode();
    }
}