/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007,2010
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * b19159       XS7.1     04/29/2010   jhanders  Add return codes for waitFor()
 */

package com.ibm.websphere.objectgrid.server;

import java.util.Collection;

import org.omg.CORBA.UserException;

import com.ibm.websphere.objectgrid.ObjectGridException;
import com.ibm.websphere.objectgrid.deployment.DeploymentPolicy;
import com.ibm.websphere.objectgrid.management.DynamicServerMBean;

/**
 * The Server is a singleton instance that represents the process scoped elements of an ObjectGrid server.
 *
 * @since WAS XD 6.1 FIX3
 */
public interface Server extends DynamicServerMBean {
    
    /**
     * Constant that represents that the server stopped normally.
     * @see #waitFor()
     * @since 7.1
     */
    int NORMAL_SERVER_STOP = 0;
    
    /**
     * Constant that represents that the server stopped due to the server no longer being connected
     * to its catalog server.
     * @see #waitFor()
     * @since 7.1
     */
    int DISCONNECTED_SERVER_STOP = 1;
    
    
    /**
     * Instantiates an ObjectGrid container capable of hosting shards for the object grids specified in the policy. The
     * policy may be built with the DeploymentPolicyFactory from xml files.
     * 
     * Precondition: The current Thread context ClassLoader contains the classes used by this container.
     *
     * @param policy
     *            The deployment policy used to build the container, may not be null.
     * @return The container reference.
     * @throws ObjectGridException
     *             If the policy can not be parsed correctly.
     *     
     * @see Thread#setContextClassLoader(java.lang.ClassLoader)
     */
    public Container createContainer(DeploymentPolicy policy) throws ObjectGridException;

    /**
     * Blocks until this server has been shutdown.
     *
     * @return The return code indicating the result of the stop.
     * @see #NORMAL_SERVER_STOP
     * @see #DISCONNECTED_SERVER_STOP
     */
    public int waitFor();
    
    
    /**
     * Retrieves a Collection of active Container Objects belonging to this Server
     * 
     * @return A Collection of active Container Objects belonging to this Server.
     * 
     * @since 7.0.0.0 FIX1
     */
    public Collection getContainers();

}
