// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.1 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/ActiveLongStatistic.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 18:08:02
//
//
// Module  :  ActiveLongStatistic.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d338878      WASX.XD   01/30/2006   jitang    Provide Transaction per second (TPS) statistic
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

/**
 * <p>This interface is used for gathering long type statistic data for a 
 * certain time period.
 * 
 * <p>This statistic has the capability to tell what the statistic data are for 
 * the last time period. For example, the number of requests in the last period. 
 * What the statistic data are is up to the implementation. 
 * 
 * @since WAS XD 6.0.1
 */
public interface ActiveLongStatistic {

    /**
     * Gets the length of the time period in milliseconds.
     * 
     * @return   the argument that was passed to the 
     *           <code>setPeriodLength(long)</code> method of
     *           this interface or <code>0</code> if <code>setPeriodLength</code> 
     *           was not previously called for this object.
     * 
     * @see #setPeriodLength(long)
     */
    long getPeriodLength();

    /**
     * Sets the length of the time period in milliseconds.
     * 
     * @param millisec the length of the time period
     */
    void setPeriodLength(long millisec);

    /**
     * Gets the data from the last period
     * 
     * @return the data from the last period
     */
    long getDataFromLastPeriod();

    /**
     * Gets the minimum value of the data for all periods.
     * 
     * @return the minimum value of the data
     */
    long getMinDataPerPeriod();

    /**
     * Gets the maximum value of the data for all periods
     * 
     * @return the maximum value of the data
     */
    long getMaxDataPerPeriod();
}


