// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006,2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  AgentStatsModule.java
//
// Source File Description:
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 13570         XD6105    12-4-2008   bsandman  code drop
// ------------ --------- ------------ --------- -----------------------------------------


package com.ibm.websphere.objectgrid.stats;

import com.ibm.websphere.objectgrid.management.AgentManagerMBean;

/**
 * <p>This interface represents a statistic module for an Agent. An Agent
 * statistic module contains the following statistics:
 *
 * <ol>
 * <li>agent class serialization time
 * <li>agent class inflation time
 * <li>failure count
 * <li>invocation count
 * <li>partition count
 * <li>reduce time
 * <li>result set inflation time
 * <li>result set serialization time
 * <li>total run time
 * </ol>
 *
 * The path to a map statistic module is the ObjectGrid name,
 * concatenated with "/", map name, and then the agent class name. You can also use an array
 * to represent the path, with the first element being the ObjectGrid name,
 * the second element being the map name, and the third being the agent class name.
 *
 * @since WAS XD 6.1.0.5
 *
 * @see AgentManagerMBean#retrieveStatsModule(String)
 */
public interface AgentStatsModule extends StatsModule {

	/**
     * Gets the reduce time statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the reduce time, time statistic
     *
     * @see ActiveTimeStatistic
     */
    ActiveTimeStatistic getReduceTime(boolean copyMode);

    /**
     * Gets the total duration time statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the total duration time statistic
     *
     * @see ActiveTimeStatistic
     */

    ActiveTimeStatistic getTotalDurationTime(boolean copyMode);

    /**
     * Gets the agent serialization time statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the agent serialization time statistic
     *
     * @see ActiveTimeStatistic
     */

    ActiveTimeStatistic getAgentSerializationTime(boolean copyMode);

    /**
     * Gets the agent inflation time statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the agent inflation time statistic
     *
     * @see ActiveTimeStatistic
     */

    ActiveTimeStatistic getAgentInflationTime(boolean copyMode);

    /**
     * Gets the result inflation time statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the result inflation time statistic
     *
     * @see ActiveTimeStatistic
     */

    ActiveTimeStatistic getResultInflationTime(boolean copyMode);

    /**
     * Gets the result serialization time statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the result serialization time statistic
     *
     * @see ActiveTimeStatistic
     */

    ActiveTimeStatistic getResultSerializationTime(boolean copyMode);

    /**
     * Gets the partition count statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the partition count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getPartitionCount(boolean copyMode);

    /**
     * Gets the failure count statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the failure count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getFailureCount(boolean copyMode);

    /**
     * Gets the invocation count statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the invocation count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getInvocationCount(boolean copyMode);

}
