// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.2 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/OGStatsModule.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:10
//
// Module  :  OGStatsModule.java
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M2_B.x WASX.XD   09/08/2005   jitang    ObjectGrid - Statistics enablement for 6.0.1
// d346485      WXD601.XD 02/10/2006   jitang    Add getAllTypeTransactionTime method
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.management.ObjectGridMBean;

/**
 * This interface represents a statistic module for an ObjectGrid. An ObjectGrid
 * statistic module contains a Session transaction time statistic. The transaction
 * time is the amount of time between when a transaction is started and committed.
 * <p>
 * The path to an ObjectGrid statistic module is the string "root",
 * concatenated with "/", and then the ObjectGrid name. You can also use an array
 * to represent the path, with the first element being the string "root",
 * and the second element being the ObjectGrid name.
 *
 * @since WAS XD 6.0.1
 *
 * @see ObjectGridMBean#retrieveStatsModule()
 * @see Session
 * @see Session#begin()
 * @see Session#commit()
 */
public interface OGStatsModule extends StatsModule {

    /**
     * Retrieves the transaction response time statistic for committed transactions for 
     * a specific transaction type.  
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param type the transaction type
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the transaction response time statistic for committed transactions for 
     * a specific transaction type
     *
     * @see ActiveTimeStatistic
     * @see Session#setTransactionType(String)
     * @see Session#TRANSACTION_TYPE_DEFAULT
     */
    ActiveTimeStatistic getTransactionTime(String type, boolean copyMode);

    /**
     * Retrieves the transaction response time statistic for all committed transactions.
     *
     * @return the combined transaction response time statistic for all committed transactions
     *
     * @see ActiveTimeStatistic
     */
    ActiveTimeStatistic getAllTypeTransactionTime();
}