// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006,2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  QueryStatsModule.java
//
// Source File Description:
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 13570         XD6105    12-4-2008   bsandman  code drop
// ------------ --------- ------------ --------- -----------------------------------------


package com.ibm.websphere.objectgrid.stats;

import com.ibm.websphere.objectgrid.management.QueryManagerMBean;

/**
 * <p>This interface represents a statistic module for a Query. A Query
 * statistic module contains 5 statistics:
 *
 * <ol>
 * <li>plan creation time
 * <li>execution count
 * <li>execution time
 * <li>failure count
 * <li>result count
 * </ol>
 *
 * The path to a map statistic module is the ObjectGrid name,
 * concatenated with "/", and then the exact query String. You can also use an array
 * to represent the path, with the first element being the ObjectGrid
 * name and the second element being the query String.
 *
 * @since WAS XD 6.1.0.5
 *
 * @see QueryManagerMBean#retrieveStatsModule(String)
 */
public interface QueryStatsModule extends StatsModule {

	/**
     * Gets the plan creation time statistic for this query.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the plan creation time statistic
     *
     * @see ActiveTimeStatistic
     */

    ActiveTimeStatistic getPlanCreationTime(boolean copyMode);

    /**
     * Gets the execution time statistic for this query.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the execution time statistic
     *
     * @see ActiveTimeStatistic
     */

    ActiveTimeStatistic getQueryExecutionTime(boolean copyMode);

    /**
     * Gets the execution count statistic for this query.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the execution count statistic
     *
     * @see ActiveCountStatistic
     */

    ActiveCountStatistic getQueryExecutionCount(boolean copyMode);

    /**
     * Gets the result count statistic for this query.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the result count statistic
     *
     * @see ActiveCountStatistic
     */

    ActiveCountStatistic getQueryResultCount(boolean copyMode);

    /**
     * Gets the failure count statistic for this query.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the failure count statistic
     *
     * @see ActiveCountStatistic
     */

    ActiveCountStatistic getQueryFailureCount(boolean copyMode);

}
