// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.1 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/ServerStatsModule.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 18:08:06
//
// Module  :  ServerStatsModule.java
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d338878      WASX.XD   01/31/2006   jitang    provide request processing time stats
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

/**
 * This interface represents a statistic module for HTTP sessions in an ObjectGrid server.
 *
 * @since WAS XD 6.0.1
 */
public interface SessionStatsModule extends StatsModule {

    /**
     * Get the session ID.
     *
     * @return the session ID
     */
    String getSessionID();

    /**
     * Set the session ID.
     *
     * @param id session ID.
     *
     */
    void setSessionID(String id);

    /**
     * Get the created sessions statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of created sessions count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getCreatedSessions(boolean copyMode);

    /**
     * Set the created sessions statistic.
     *
     * @param stat number of created sessions.
     */
    void setCreatedSessions(long stat);

    /**
     * Get the invalidated sessions statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of invalidated sessions count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getInvalidatedSessions(boolean copyMode);

    /**
     * Set the invalidated sessions statistic.
     *
     * @param stat number of invalidated sessions.
     */
    void setInvalidatedSessions(long stat);

    /**
     * Get the active sessions statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of active sessions count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getActiveSessions(boolean copyMode);

    /**
     * Set the active sessions statistic.
     *
     * @param stat number of active sessions.
     */
    void setActiveSessions(long stat);

    /**
     * Get the sessions in memory count statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of sessions in memory count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getMemoryCount(boolean copyMode);

    /**
     * Set the sessions in memory count statistic.
     *
     * @param stat number of sessions in memory.
     */
    void setMemoryCount(long stat);

    /**
     * Get the discarded sessions statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of discarded sessions count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getCacheDiscards(boolean copyMode);

    /**
     * Set the discarded sessions statistic.
     *
     * @param stat number of discarded sessions.
     */
    void setCacheDiscards(long stat);

    /**
     * Get the affinity breaks statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of affinity breaks count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getAffinityBreaks(boolean copyMode);

    /**
     * Set the affinity breaks statistic.
     *
     * @param stat number of affinity breaks.
     */
    void setAffinityBreaks(long stat);

    /**
     * Get the sessions invalidated by timeout statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of sessions invalidated by timeout count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getInvalidatedByTimeout(boolean copyMode);

    /**
     * Set the sessions invalidated by timeout statistic.
     *
     * @param stat number of sessions invalidated by timeout.
     */
    void setInvalidatedByTimeout(long stat);

    /**
     * Get the access to nonexistent sessions statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of accesses to nonexistent sessions count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getAccessToNonExistentSession(boolean copyMode);

    /**
     * Set the access to nonexistent sessions statistic.
     *
     * @param stat number of accesses to nonexistent sessions.
     */
    void setAccessToNonExistentSession(long stat);

    /**
     * Get the access to sessions statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of sessions accessed count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getAccessedSessions(boolean copyMode);

    /**
     * Set the access to sessions statistic.
     *
     * @param stat number of accessed sessions.
     */
    void setAccessedSessions(long stat);
}
