// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  OneToMany.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b995         6.1       05/22/2006   cdjohnson New part.
// b2404        XD 6.1    09/20/2006   cdjohnson Feature updates for M6
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.annotations;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Specifies a multi-valued association to another entity that has one-to-many
 * multiplicity.
 * 
 * @since WAS XD 6.1
 */
@Target({FIELD, METHOD})
@Retention(RUNTIME)
public @interface OneToMany {
    /**
     * The alternative name for this attribute.  The alias can be used to map
     * one entity to another or override the default property name behavior
     * in legacy objects. It can also be used to refer to this attribute in queries.
     * @return the alias name.
     */
    String alias() default "";
    
    /**
     * The target class of the association.  Required only
     * if the target class type cannot be inferred by the type of field or property
     * that stores the association. 
     * @return the target class or void.class if not specified.
     */
	Class targetEntity() default void.class;

    /**
     * The field that owns the relationship. The mappedBy element is only specified on the
     * inverse (non-owning) side of the association.  Required only when
     * the owner of the relationship cannot be inferred.
     * @return the field that owns the relationship.
     */
    String mappedBy() default "";

    /**
     * Specifies which operations must be cascaded to the target of the association.  
     * By default, no operations are cascaded.
     * @return the operations to cascade.
     */
    CascadeType[] cascade() default {};
    

    /**
     * Suggestion to the projector on how to retrieve the data for this 
     * association.<p>
     * 
     * The <tt>EAGER</tt> strategy instructs the projector to eagerly
     * fetch the data.  The <tt>LAZY</tt> strategy is a <i>hint</i> to the 
     * projector to lazily fetch the data when it is first accessed.<p>
     * 
     * @return the FetchType hint.
     */
    FetchType fetch() default FetchType.LAZY;    

}
