// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  FetchType.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b2405        6.1       11/14/2006   rajeshk New part
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.md;

import java.io.Serializable;

/**
 * Defines strategies for fetching data from the TupleStore.<p>
 * 
 * The <tt>EAGER</tt> strategy instructs the projector to eagerly
 * fetch the data.  The <tt>LAZY</tt> strategy is a <i>hint</i> to the 
 * projector to lazily fetch the data when it is first accessed.<p>
 * 
 * @see TupleAssociation
 * @since WAS XD 6.1
 */
public final class FetchType implements Serializable {    

    private static final long serialVersionUID = -9172617647136454893L;

    /**
     * Fetch the data lazily.
     */
    public final static FetchType LAZY = new FetchType(0);

    /**
     * Fetch the data eagerly.
     */
    public final static FetchType EAGER = new FetchType(1);

    /**
     * The method in which the data is fetched is undefined.
     */
    public final static FetchType UNDEFINED = new FetchType(2);

    private final static FetchType[] PRIVATE_VALUES = new FetchType[] { LAZY, EAGER, UNDEFINED };

    /**
     * Id that uniquely identifies the AccessType instance.
     */
    final private int fetchType;

    /**
     * Hidden CTOR for enumerated type idiom to use.
     * 
     * @param fetchType
     *            is the unique id for the contructed FetchType instance.
     */
    private FetchType(int fetchType) {
        this.fetchType = fetchType;
    }

    /**
     * Hidden default CTOR for enumerated type idiom to use.
     */
    private FetchType() {
        this(0);//LAZY
    }

    /**
     * Returns a string representation of the <code>FetchType</code>.
     * 
     * @return a string representation of the <code>FetchType</code>.
     */
    public String toString() {
        switch (fetchType) {
        case 0:
            return "LAZY"; //$NON-NLS-1$
        case 1:
            return "EAGER"; //$NON-NLS-1$
        case 2:
            return "UNDEFINED"; //$NON-NLS-1$
        default:
            return "UNKNOWN: " + fetchType; //$NON-NLS-1$
        }
    }
   
    /**
     * Retreive the value of this FetchType.  Can be useful when 
     * using with a switch statement.
     * @return the int value of this FetchType. 
     */
    public FetchType getFetchType() {
        switch (fetchType) {
        case 0:
            return LAZY;
        case 1:
            return EAGER;
        default:
            return UNDEFINED;
        }

    }
    

    /*
     * Required to avoid creating a different object instance each time the object is deserialized.
     */
    private Object readResolve() {
        return PRIVATE_VALUES[fetchType];
    }

    /**
     * Retrieves the hash code of this FetchType object.
     * @return the hash code value of this object.
     */
    public int hashCode() {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + fetchType;
        return result;
    }

    /**
     * Returns true if the input object is equivalent to
     * this FetchType.
     * 
     * @param obj the object with which to compare.
     * @return true if the input Object is equivalent to this FetchType
     */
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final FetchType other = (FetchType) obj;
        if (fetchType != other.fetchType)
            return false;
        return true;
    }
}
