//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// wi906       8.5       02/29/2012   cdjohnson new part
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.xs.ra;

/**
 * Allows administration clients control the lifecycle of the eXtreme Scale resource adapter's connection.</p>
 * 
 * The object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=ObjectGridJ2CConnection,objectGridName=&lt;objectgrid&gt;,domain=&lt;domain name&gt;,connectionName=&lt;connection name&gt;
 * </pre>
 * 
 * Note:  Additional properties may be included.
 * @since 8.5
 * @since XC10
 */
public interface ObjectGridJ2CConnectionMBean {
    
    /**
     * The MBean type
     */
    public String MBEAN_TYPE = "ObjectGridJ2CConnection";
    
    /**
     * The connection status indicating that a connection has been established.
     */
    public String CONNECTIONSTATUS_CONNECTED = "CONNECTED";

    /**
     * The connection status indicating that client is no longer connected.
     */
    public String CONNECTIONSTATUS_DISCONNECTED = "DISCONNECTED";
    
    /**
     * The default connection name prefix when not specified.
     */
    public String CONNECTIONNAME_DEFAULT = "DEFAULT";
    
    /**
     * Reset the ObjectGrid connection.<p>
     * 
     * This destroys the client connection to the ObjectGrid, including any 
     * local cache that may be created.<p>  
     * 
     * Subsequent uses of the ManagedConnectionFactory will result in a new ObjectGrid connection. 
     */
    public void resetObjectGridConnection();
    
    /**
     * The name of the ObjectGrid that is connected.
     * @return the ObjectGrid name
     */
    public String getObjectGridName();
    
    /**
     * The name of the connection as specified on the J2C ConnectionFactory, or "DEFAULT" if
     * not specified.  If there are multiple connections with the same attributes in the
     * same process, the "DEFAULT" name will have an integer appended.
     * @return the connection name.
     */
    public String getConnectionName();
    
    /**
     * The domain name of the catalog service domain as reported by the catalog service.
     * @return the catalog service domain name.
     */
    public String getDomainName();
    
    /**
     * The catalog service endpoints of the catalog service.
     * @return the catalog service endpoints 
     */
    public String getCatalogServiceEndpoints();
    
    /**
     * The status of the connection.  Valid states include "CONNECTED" or "DISCONNECTED".<p>
     * 
     * Other states may be returned in the future.
     * @return one of the CONNECTIONSTATUS constants defined in this interface.
     */
    public String getConnectionStatus();
}
