//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// wi906       8.5       02/29/2012   cdjohnson new part
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.xs.ra;

import java.io.PrintWriter;
import java.util.Set;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterAssociation;
import javax.security.auth.Subject;

import com.ibm.ws.xs.ra.spi.XSMConFactoryImpl;

/**
 * The XSManagedConnectionFactory create managed connections to the eXtreme Scale ObjectGrids.<p>
 * @since 8.5
 * @since XC10
 */
public final class XSManagedConnectionFactory 
implements ManagedConnectionFactory, ResourceAdapterAssociation {
	private static final long serialVersionUID = 7832078167887569470L;
	
	private XSMConFactoryImpl xsMcf;

	/*
     * Instantiates the underlying impl object. 
     * Warning: do not call this directly from the constructor otherwise WAS introspection on BASE 
     * installations will fail.
     */
	private void instantiateImpl() {
		if (xsMcf == null) {
		    xsMcf = new XSMConFactoryImpl(this);
		}
	}
	
	/**
	 * Retrieve the associated eXtreme Scale resource adapter instance.
	 * @return The associated eXtreme Scale resource adapter
	 * @see ResourceAdapterAssociation#getResourceAdapter()
	 */
	public ResourceAdapter getResourceAdapter() {
		instantiateImpl();
		return ((ResourceAdapterAssociation)xsMcf).getResourceAdapter();
	}

	/**
	 * Associate this object with an eXtreme Scale resource adapter.
	 * Note, this method must be called exactly once. That is, the association must not change during the lifetime of this object.
	 * @param ra the eXtreme Scale resource adapter  
	 * @see ResourceAdapterAssociation#setResourceAdapter(ResourceAdapter)
	 * @throws ResourceException
	 * @throws IllegalStateException
	 */
	public void setResourceAdapter(ResourceAdapter ra) throws ResourceException {
		instantiateImpl();
		((ResourceAdapterAssociation)xsMcf).setResourceAdapter(ra);
	}


	/**
	 * Creates a non-managed XSConnectionFactory instance. 
	 * The XSConnectionFactory instance gets initialized with a default XSConnectionManager instance provided by the 
	 * eXtreme Scale resource adapter.
	 * @return the new XSConnectionFactory instance
	 * @see ManagedConnectionFactory#createConnectionFactory()
	 * @throws ResourceException
	 */
	public Object createConnectionFactory() throws ResourceException {
		instantiateImpl();
		return xsMcf.createConnectionFactory();
	}
	/**
	 * Creates a managed XSConnectionFactory instance. The XSConnectionFactory instance gets initialized with the 
	 * passed ConnectionManager. In the managed scenario, ConnectionManager is provided by the application server.
	 * @param conMgr - Connection Manager to be associated with the XSConnectionFactory.
	 * @return the new XSConnectionFactory instance
	 * @see ManagedConnectionFactory#createConnectionFactory(ConnectionManager)
	 * @throws ResourceException
	 * 		   ResourceAdapterInternalException 
	 */
	public Object createConnectionFactory(ConnectionManager conMgr)
			throws ResourceException {
		instantiateImpl();
		return xsMcf.createConnectionFactory(conMgr);
	}

	/**
	 * Creates a new physical connection to the underlying ObjectGrid
	 * @param subject - Caller's security information. WebSphere eXtreme Scale client uses
	 * the credentials specified in a CredentialGenerator as part of the ConnectionSpec
	 * or the client properties.
	 * @param conReq - XS specific connection properties
	 * @return the new XSManagedConnection instance
	 * @see ManagedConnectionFactory#createManagedConnection(Subject, ConnectionRequestInfo)
	 * @throws ResourceException
	 */
	public ManagedConnection createManagedConnection(Subject subject,
			ConnectionRequestInfo conReq) throws ResourceException {
		instantiateImpl();
		return xsMcf.createManagedConnection(subject, conReq);
	}

	/**
	 * Get the log writer for this XSManagedConnectionFactory instance. 
	 * @return PrintWriter
	 * @see ManagedConnectionFactory#getLogWriter()
	 * @throws ResourceException
	 */
	public PrintWriter getLogWriter() throws ResourceException {
		instantiateImpl();
		return xsMcf.getLogWriter();
	}

	/**
	 * Returns a matched connection from the candidate set of connections. 
	 * @param connectionSet - candidate connection set
	 * @param subject - caller's security information
	 * @param cxRequestInfo - XS specific connection properties 
	 * @return XSManagedConnection instance if acceptable match found otherwise null
	 * @see ManagedConnectionFactory#matchManagedConnections(Set, Subject, ConnectionRequestInfo)
	 * @throws ResourceException
	 */
	public ManagedConnection matchManagedConnections(@SuppressWarnings("rawtypes") Set connectionSet, 
	                                                 Subject subject,
			ConnectionRequestInfo cxRequestInfo) throws ResourceException {
		instantiateImpl();
		return xsMcf.matchManagedConnections(connectionSet, subject, cxRequestInfo);
	}
	
	/**
	 * Set the log writer for this ManagedConnectionFactory instance
	 * @param out - PrintWriter - an out stream for error logging and tracing
	 * @see ManagedConnectionFactory#setLogWriter(PrintWriter) 
	 * @throws ResourceException
	 */
	public void setLogWriter(PrintWriter out) throws ResourceException {
		instantiateImpl();
		xsMcf.setLogWriter(out);
	}

	/**
	 * Destroy this MCF and release any other resources.
	 */
	public void destroy() {
		if (xsMcf != null) {
			xsMcf.destroy();
			xsMcf = null;
		}
	}

	/**
	 * Get the eXtreme Scale specific property: ConnectionName
	 * @return The name of the eXtreme Scale client connection
	 */
	public String getConnectionName() {
		instantiateImpl();
		return xsMcf.getConnectionName();
	}

	/**
	 * Set the eXtreme Scale specific property: ConnectionName
	 * @param connName - The name of the eXtreme Scale client connection
	 */
	public void setConnectionName(String connName) {
		instantiateImpl();
		xsMcf.setConnectionName(connName);
	}

	/**
	 * Get the eXtreme Scale specific property: ObjectGridName
	 * @return The data grid name
	 */
	public String getObjectGridName() {
		instantiateImpl();
		return xsMcf.getObjectGridName();
	}

	/**
	 * Set the eXtreme Scale specific property: ObjectGrid Name
	 * @param objectGridName - The data grid name
	 */
	public void setObjectGridName(String objectGridName) {
		instantiateImpl();
		xsMcf.setObjectGridName(objectGridName);
	}

    
	/**
	 * Get the eXtreme Scale specific property: CatalogServiceEndpoints
	 * @return The catalog service domain end points
	 */
	public String getCatalogServiceEndpoints() {
		instantiateImpl();
		return xsMcf.getCatalogServiceEndpoints();
	}
	
	/**
	 * Set the eXtreme Scale specific property: CatalogServiceEndpoints
	 * @param catalogServiceEndpoints - The catalog service domain end points
	 */
	public void setCatalogServiceEndpoints(String catalogServiceEndpoints) {
		instantiateImpl();
		xsMcf.setCatalogServiceEndpoints(catalogServiceEndpoints);
	}

	/**
	 * Get the eXtreme Scale specific property: CatalogDomain
	 * @return The catalog service domain name defined in WebSphere Application Server
	 */
	public String getCatalogServiceDomain() {
		instantiateImpl();
		return xsMcf.getCatalogServiceDomain();
	}
	
	/**
	 * Set the eXtreme Scale specific property: CatalogDomain
	 * @param catalogServiceDomain - The catalog service domain name defined in WebSphere Application Server
	 */
	public void setCatalogServiceDomain(String catalogServiceDomain) {
		instantiateImpl();
		xsMcf.setCatalogServiceDomain(catalogServiceDomain);
	}

	/**
	 * Get the eXtreme Scale specific property: ObjectGridURL
	 * @return The URL of the client data grid override XML file
	 */
	public String getObjectGridURL() {
		instantiateImpl();
		return xsMcf.getObjectGridURL();
	}
	
	/**
	 * Set the eXtreme Scale specific property: ObjectGridURL
	 * @param objectGridURL The URL of the client data grid override XML file
	 */
	public void setObjectGridURL(String objectGridURL) {
		instantiateImpl();
		xsMcf.setObjectGridURL(objectGridURL);
	}
	
	/**
	 * Get the eXtreme Scale specific property: ObjectGridResource
	 * @return The resource path of the client data grid override XML file
	 */
	public String getObjectGridResource() {
		instantiateImpl();
		return xsMcf.getObjectGridResource();
	}
	
	/**
	 * Set the eXtreme Scale specific property: ObjectGridResource
	 * @param objectGridResource - The resource path of the client data grid override XML file
	 */
	public void setObjectGridResource(String objectGridResource) {
		instantiateImpl();
		xsMcf.setObjectGridResource(objectGridResource);
	}

	/**
	 * Get the eXtreme Scale specific property: ClientPropertiesURL
	 * @return The URL of the client properties file
	 */
	public String getClientPropertiesURL() {
		instantiateImpl();
		return xsMcf.getClientPropertiesURL();
	}
	
	/**
	 * Set the eXtreme Scale specific property: ClientPropertiesURL
	 * @param clientPropsURL - The URL of the client properties file
	 */
	public void setClientPropertiesURL(String clientPropsURL) {
		instantiateImpl();
		xsMcf.setClientPropertiesURL(clientPropsURL);
	}

	/**
	 * Get the eXtreme Scale specific property: ClientPropertiesResource
	 * @return The resource path of the client properties file
	 */
	public String getClientPropertiesResource() {
		instantiateImpl();
		return xsMcf.getClientPropertiesResource();
	}

	/**
	 * Get the eXtreme Scale specific property: ClientPropertiesResource
	 * @param clientPropsResource - The resource path of the client properties file
	 */
	public void setClientPropertiesResource(String clientPropsResource) {
		instantiateImpl();
		xsMcf.setClientPropertiesResource(clientPropsResource);
	}
	
    /**
     * Warning: This method is used via reflection. Do not change its signature without updating the reflection code.
     * The reflection lookup code is called from:
     *   - The unit test code in wxs.ra.unittest.web/CommonTester
     */
	@SuppressWarnings("unused")
	private ManagedConnectionFactory getImpl() {
	    instantiateImpl();
		return xsMcf;
	}
	
	/**
     * Set the eXtreme Scale specific property: LocalGrid
     * @param useLocal - Indicates if this managed connection factory is used only for access to a local ObjectGrid instance
     */
    public void setLocalGrid(boolean useLocal) {
        instantiateImpl();
        xsMcf.setLocalGrid(useLocal);
    }
    
    /**
     * Get the eXtreme Scale specific property: LocalGrid
     * @return Is this managed connection factory used only for access to a local ObjectGrid instance?
     */
    public boolean isLocalGrid() {
        instantiateImpl();
        return xsMcf.isLocalGrid();
    }

	/**
	 * @see ManagedConnectionFactory#hashCode()
	 */
	@Override
	public int hashCode() {
		instantiateImpl();
		return xsMcf.hashCode();
	}

	/**
	 * @see ManagedConnectionFactory#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		instantiateImpl();
		if (obj instanceof XSManagedConnectionFactory) {
		    return xsMcf.equals(((XSManagedConnectionFactory)obj).xsMcf);
		}
		return false;
	}
	
    public String toString() {
        return super.toString() + "[" + (xsMcf == null ? "none" : xsMcf.toString() ) + "]";   
    }

}
