// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.

package com.ibm.websphere.xs.rest;

/**
 * This interface is implemented by object used in WebSphere eXtreme Scale REST Gateway.   
 * Such object is used as a data wrapper for data to be accessed via WebSphere eXtreme Scale REST Gateway.
 * With its wrapped data, it will be stored in an object grid's map entry, with a
 * String as the key of the map entry.
 * 
 * @since 8.6
 * @see com.ibm.websphere.xs.rest.RestValueFactory
 * 
 */
public interface RestValue {
	
	/**
	 * "application/xml" HTML content type
	 * 
	 * @since 8.6
	 */
	static public String CONTENT_TYPE_APP_XML 	= "application/xml";
	
	/**
	 * "application/json" HTML content type
	 * 
	 * @since 8.6
	 */
	static public String CONTENT_TYPE_APP_JSON	= "application/json";
	
	/**
	 * "application/octet-stream" HTML content type
	 * 
	 * @since 8.6
	 */
	static public String CONTENT_TYPE_APP_OCTET = "application/octet-stream";

	
	/**
     * Return content type of the contained value
     * 
     * @since 8.6
     * @return
     * 		A content type
     */
	public abstract String getContentType();

	/**
     * Return contained value
     * 
     * @since 8.6
     * @return 
     * 		An array of byte that represents the contained value
     */
	public abstract byte[] getValue();

}
